/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2009 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.dita;

import java.util.Stack;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.extensions.api.link.Attr;
import ro.sync.ecss.extensions.api.link.ElementLocator;

/**
 * An implementation for a DITA element when the referred element is not a topic. So the link
 * has the following pattern: topicID/elementID
 */
@API(type=APIType.INTERNAL, src=SourceType.PUBLIC)
public class DITAElementLocator extends ElementLocator {
  
  /**
   * The ID of the parent topic element.
   */
  private String topicId;
  
  /**
   * The ID of the element inside the topic.
   */
  private String elementId;
  
  /**
   * Stores all the elements we have entered in.
   */
  private Stack elementStack = new Stack(); 

  /**
   * Constructor.
   * 
   * @param link The link used to identify the element.
   */
  public DITAElementLocator(String link) {
    super(link);
    String[] tokens = link.split("/");
    topicId = tokens[0];
    elementId = tokens[1];
  }

  /**
   * @see ro.sync.ecss.extensions.api.link.ElementLocator#endElement(java.lang.String, java.lang.String, java.lang.String)
   */
  @Override
  public void endElement(String uri, String localName, String name) {
    if (!elementStack.isEmpty()) {
      elementStack.pop();
    }
  }

  /**
   * @see ro.sync.ecss.extensions.api.link.ElementLocator#startElement(java.lang.String, java.lang.String, java.lang.String, ro.sync.ecss.extensions.api.link.Attr[])
   */
  @Override
  public boolean startElement(String uri, String localName, String name, Attr[] atts) {
    boolean toReturn = false;
    String idAttribute = getIdValue(atts);
    boolean isTopic = isTopic(atts);
    if (!isTopic && elementId.equals(idAttribute)) {
      // This is the searched element. Check for topic ancestor in element stack.
      int length = elementStack.size();
      for (int i = length - 1; i >= 0; i--) {
        Attr[] elAttrs = (Attr[]) elementStack.get(i);
        if (isTopic(elAttrs)) {
          // This is the first topic.
          if (topicId.equals(getIdValue(elAttrs))) {
            // The first topic has the searched id.
            toReturn = true;
          }
          // The first parent topic is important.
          break;
        }
      }
    }
    
    elementStack.push(atts);
    return toReturn;
  }

  /**
   * Check if the element with the given attributes is a topic.
   * 
   * @param atts The attributes of the element.
   * 
   * @return <code>true</code> if the element is a topic.
   */
  private boolean isTopic(Attr[] atts) {
    boolean isTopic = false;
    for (int i = 0; i < atts.length; i++) {
      if ("class".equals(getLocalName(atts[i].getQName()))) {
        // The class attribute.
        isTopic = atts[i].getValue().contains(" topic/topic ");
        break;
      }
    }
    return isTopic;
  }
  
  /**
   * Get the value of the id attribute, if present.
   * 
   * @param atts The attributes of the element.
   * 
   * @return The value of the id attribute or <code>null</code> if not present.
   */
  private String getIdValue(Attr[] atts) {
    String id = null;
    for (int i = 0; i < atts.length; i++) {
      if ("id".equals(getLocalName(atts[i].getQName()))) {
        id = atts[i].getValue();
        break;
      }
    }
    return id;
  }

  /**
   * Extract local part for qualified name.
   */
  private String getLocalName(String qName) {
    String local = qName;
    if(qName != null){
      int idx = qName.lastIndexOf(':');
      if(idx != -1){
        local = qName.substring(idx + 1);
      }
    }
    return local;
  }
}