/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2009 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.dita.map.table;

import javax.swing.text.BadLocationException;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperation;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.AuthorOperationStoppedByUserException;
import ro.sync.ecss.extensions.api.node.AttrValue;
import ro.sync.ecss.extensions.api.node.AuthorDocumentFragment;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;
import ro.sync.ecss.extensions.commons.table.operations.AuthorTableHelper;
import ro.sync.ecss.extensions.commons.table.operations.InsertTableOperationBase;
import ro.sync.ecss.extensions.commons.table.operations.TableInfo;
import ro.sync.ecss.extensions.commons.table.operations.TableOperationsUtil;

/**
 * Operation used to insert a DITA map reltable.
 */
@API(type=APIType.INTERNAL, src=SourceType.PUBLIC)
public class InsertTableOperation implements AuthorOperation, InsertTableOperationBase {
  
  /**
   * @see ro.sync.ecss.extensions.api.AuthorOperation#doOperation(ro.sync.ecss.extensions.api.AuthorAccess, ro.sync.ecss.extensions.api.ArgumentsMap)
   */
  @Override
  public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
      throws IllegalArgumentException, AuthorOperationException {
    insertTable(null, false, authorAccess, null, null);
  }

  /**
   * Add simple table body.
   * 
   * @param tableXMLFragment  The string buffer to append content.
   * @param tableInfo         The table info.
   * @param fragments         The cell fragments.
   * @param cellsFragments    <code>true</code> if cells.
   * @param authorAccess      The Author access.
   * @param namespace         The table namespace.
   * @param tableHelper       The table helper.
   * 
   * @throws AuthorOperationException 
   */
  private void addRelTableBody(
      StringBuilder tableXMLFragment,
      TableInfo tableInfo,
      AuthorDocumentFragment[] fragments,
      boolean cellsFragments,
      AuthorAccess authorAccess,
      String namespace,
      AuthorTableHelper tableHelper) throws AuthorOperationException {

    for (int i = 0; i < tableInfo.getRowsNumber(); i++) {
      tableXMLFragment.append("<relrow>");
      for (int j = 0; j < tableInfo.getColumnsNumber(); j++) {
        if (fragments != null) {
          String cellXMLFragment = TableOperationsUtil.createCellXMLFragment(
              authorAccess, 
              fragments, 
              cellsFragments, 
              "relcell", 
              tableInfo.isGenerateHeader() ? (i + 1) : i, 
              namespace, 
              tableHelper);
          tableXMLFragment.append(cellXMLFragment);
        } else {
          tableXMLFragment.append("<relcell></relcell>");
        }
      }
      tableXMLFragment.append("</relrow>"); 
    }
  }
  
  /**
   * Add simple table header.
   *
   * @throws AuthorOperationException 
   */
  private void addRelTableHeader(StringBuilder tableXMLFragment, TableInfo tableInfo, 
      AuthorDocumentFragment[] fragments, boolean cellsFragments, AuthorAccess authorAccess, 
      String namespace, AuthorTableHelper tableHelper) throws AuthorOperationException {
    if (tableInfo.isGenerateHeader()) {
      tableXMLFragment.append("<relheader>");
      for (int i = 1; i <= tableInfo.getColumnsNumber(); i++) {
        if (i == 1 && fragments != null && fragments.length > 0) {
          String cellXMLFragment = TableOperationsUtil.createCellXMLFragment(
              authorAccess, 
              fragments, 
              cellsFragments, 
              "relcolspec", 
              0, 
              namespace, 
              tableHelper,
              (String[]) null);
          tableXMLFragment.append(cellXMLFragment);
        } else {
          tableXMLFragment.append("<relcolspec/>");
        }
      }
      tableXMLFragment.append("</relheader>");
    }
  }

  /**
   * No arguments. The operation will display a dialog for choosing the table attributes.
   * 
   * @see ro.sync.ecss.extensions.api.AuthorOperation#getArguments()
   */
  @Override
  public ArgumentDescriptor[] getArguments() {
    return null;
  }

  /**
   * @see ro.sync.ecss.extensions.api.Extension#getDescription()
   */
  @Override
  public String getDescription() {
    return "Insert a DITA map relationship table";
  }

  /**
   * @see ro.sync.ecss.extensions.commons.table.operations.InsertTableOperationBase#insertTable(ro.sync.ecss.extensions.api.node.AuthorDocumentFragment[], boolean, ro.sync.ecss.extensions.api.AuthorAccess, java.lang.String, ro.sync.ecss.extensions.commons.table.operations.AuthorTableHelper)
   */
  @Override
  public void insertTable(AuthorDocumentFragment[] fragments, boolean cellsFragments,
      AuthorAccess authorAccess, String namespace, AuthorTableHelper tableHelper)
      throws AuthorOperationException {
    // Show the 'Insert table' dialog
    int rowsCount = 0;
    int columnsCount = 0;
    if (fragments != null) {
      rowsCount = fragments.length;
      columnsCount = 1;
    }
    TableInfo tableInfo = null;
    if(authorAccess.getWorkspaceAccess().isStandalone()) {
      tableInfo = SADITARelTableCustomizer.getInstance().customizeTable(authorAccess, rowsCount, columnsCount);
    } else {
      //Eclipse table customization
      tableInfo = ECDITARelTableCustomizer.getInstance().customizeTable(authorAccess, rowsCount, columnsCount);
    }

    if (tableInfo != null) {
      // Create the table XML fragment
      StringBuilder tableXMLFragment = new StringBuilder();
      if(tableInfo.getTitle() == null || tableInfo.getTitle().trim().length() == 0) {
        tableXMLFragment.append("<reltable>");
      } else {        
        tableXMLFragment.append("<reltable title=\"" + tableInfo.getTitle() + "\">");
      }

      // Add table header
      addRelTableHeader(tableXMLFragment, tableInfo, fragments, cellsFragments, authorAccess, namespace, tableHelper);

      // Add table body
      addRelTableBody(tableXMLFragment, tableInfo, fragments, cellsFragments, authorAccess, namespace, tableHelper);

      tableXMLFragment.append("</reltable>");

      // EXM-24925 Place the caret in the closest map element, to the right
      int caretOffset = authorAccess.getEditorAccess().getCaretOffset();
      int offsetToInsert = caretOffset;
      try {
        AuthorNode currentNode = authorAccess.getDocumentController().getNodeAtOffset(caretOffset);
        if (currentNode instanceof AuthorElement) {
          AuthorElement currentElement = (AuthorElement) currentNode;
          AttrValue classAttribute = currentElement.getAttribute("class");
          String classAttributeValue = classAttribute.getValue();
          if (classAttributeValue.contains(" map/map ")) {
            // We are already in a map element. Insert at current caret offset.
          } else {
            while (currentElement.getParentElement() != null) {
              AuthorElement parentElement = (AuthorElement) currentElement.getParentElement();
              classAttribute = parentElement.getAttribute("class");
              classAttributeValue = classAttribute.getValue();
              if (classAttributeValue.contains(" map/map ")) {
                // The parent node is a map. The insertion point will be after the current node offset.
                offsetToInsert = currentElement.getEndOffset() + 1;
                break;
              }
              currentElement = parentElement;
            }
          }
        }
      } catch (BadLocationException e) {
        // Nothing to do
      }
      
      // Insert the table 
      authorAccess.getDocumentController().insertXMLFragmentSchemaAware(
          tableXMLFragment.toString(), 
          offsetToInsert);
    } else {
      // User canceled the operation 
      throw new AuthorOperationStoppedByUserException("Cancelled by user");
    }
  }
}