/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.docbook;

import java.util.List;

import javax.swing.text.BadLocationException;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorElementBaseInterface;
import ro.sync.ecss.extensions.api.node.AuthorDocumentFragment;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;
import ro.sync.xml.XmlUtil;

/**
 * Utility class for promote/demote actions for Docbook.
 * 
 * @author costi
 */
@API(type=APIType.INTERNAL, src=SourceType.PUBLIC)
public class PromoteDemoteSectionUtil {
  
  /**
   * Docbook5 namespace.
   */
  private static final String DOCBOOK5_NAMESPACE = "http://docbook.org/ns/docbook";

  /**
   * Promote/demote section action.
   */
  public enum PromoteDemote {
    /**
     * Promote section action.
     */
    PROMOTE,
    /**
     * Demote section action.
     */
    DEMOTE
  }
  
  /**
   * Executes the promote/demote section action.
   * 
   * @param authorAccess The author access.
   * @param action The promote/demote action.
   */
  public static void processPromoteDemote(AuthorAccess authorAccess, PromoteDemote action) {
    // Find the section node that should be promoted/demoted
    AuthorNode sectionNodeToProcess = getSectionNodeToProcess(authorAccess);
    
    if (sectionNodeToProcess instanceof AuthorElement) {
      int insertionOffset = getInsertionOffset(authorAccess, (AuthorElement) sectionNodeToProcess, action);
      if (insertionOffset != -1) {
        AuthorElement sectionNodeElement = (AuthorElement)sectionNodeToProcess;
        String elementName = ((AuthorElement)sectionNodeToProcess).getLocalName();
        if (!"section".equals(elementName)) {
          changeSectNodeName(authorAccess, sectionNodeElement, action);
        }

        try {
          // Insert the new node.
          int caretOffset = authorAccess.getEditorAccess().getCaretOffset();
          int relativeCaretOffset = caretOffset - sectionNodeToProcess.getStartOffset();
          AuthorDocumentFragment sectionDocumentFragment =
              authorAccess.getDocumentController().createDocumentFragment(sectionNodeToProcess, true);
          authorAccess.getDocumentController().insertFragment(insertionOffset, sectionDocumentFragment);
          authorAccess.getDocumentController().deleteNode(sectionNodeToProcess);
          int nextCaretPosition = insertionOffset + relativeCaretOffset;
          if (action == PromoteDemote.PROMOTE) {
            nextCaretPosition = nextCaretPosition - sectionDocumentFragment.getLength();
          }
          authorAccess.getEditorAccess().setCaretPosition(nextCaretPosition);
        } catch (BadLocationException e) {
          // Do nothing.
        } 
      }
    }
  }
  
  /**
   * Gets the section node to be processed.
   * 
   * @param authorAccess The author access.
   * 
   * @return The section node to be processed.
   */
  private static AuthorNode getSectionNodeToProcess(AuthorAccess authorAccess) {
    AuthorElement nodeToReturn = null;
    try {
      int selectionStart = authorAccess.getEditorAccess().getSelectionStart();
      int selectionEnd = authorAccess.getEditorAccess().getSelectionEnd();
      AuthorNode startingNode = null;
      if (selectionStart == selectionEnd) {
        // No selection.
        int caretOffset = authorAccess.getEditorAccess().getCaretOffset();
        startingNode = authorAccess.getDocumentController().getNodeAtOffset(caretOffset);
      } else {
        // Get the node which includes the selection.
        List<AuthorNode> nodesToSelect = authorAccess.getDocumentController().getNodesToSelect(selectionStart, selectionEnd);
        if (nodesToSelect.size() == 1) {
          startingNode = nodesToSelect.get(0);
        } else {
          AuthorNode[] nodesArray = nodesToSelect.toArray(new AuthorNode[0]);
          startingNode = authorAccess.getDocumentController().getCommonAncestor(nodesArray);
        }
      }
      if (startingNode instanceof AuthorElement) {
        // Look for closest section node.
        nodeToReturn = (AuthorElement) startingNode;
        while ((!nodeToReturn.getLocalName().equals("section")
            && !nodeToReturn.getLocalName().startsWith("sect")) 
            || !(nodeToReturn.getNamespace().isEmpty() || nodeToReturn.getNamespace().equals(DOCBOOK5_NAMESPACE))) {

          if (nodeToReturn.getParentElement() instanceof AuthorElement) {
            nodeToReturn = (AuthorElement) nodeToReturn.getParentElement();
          } else {
            break;
          }
        }
      } 
    } catch (BadLocationException e) {
      e.printStackTrace();
    }
    
    return nodeToReturn;
  }
  
  /**
   * Get the offset where the promoted/demoted section node should be inserted. 
   * 
   * @param sectionNodeToProcess The section node that should be promoted/demoted.
   * @param action The promote/demote action.
   * 
   * @return The offset where the promoted/demoted section node should be inserted.
   */
  private static int getInsertionOffset(AuthorAccess authorAccess, AuthorElement sectionNodeToProcess, PromoteDemote action) {
    int insertionOffset = -1;

    if (action == PromoteDemote.PROMOTE) {
      // Compute insertion offset for promote action.
      if (sectionNodeToProcess.getParentElement() != null) {
        AuthorElement sectionNodeParent = (AuthorElement)sectionNodeToProcess.getParentElement();
        if (sectionNodeParent != null && sectionNodeParent.getParent() != authorAccess.getDocumentController().getAuthorDocumentNode()) {
          // The parent must not be the root.
          if ((sectionNodeToProcess.getNamespace().isEmpty() 
              || sectionNodeToProcess.getNamespace().equals(DOCBOOK5_NAMESPACE))) {
            if (sectionNodeToProcess.getLocalName().equals("section")) {
              if (sectionNodeParent.getLocalName().equals("section")
                  && (sectionNodeToProcess.getNamespace().isEmpty() || sectionNodeToProcess.getNamespace().equals(DOCBOOK5_NAMESPACE))) {
                // get the insertion offset
                insertionOffset = sectionNodeParent.getEndOffset() + 1;
              }
            } else {
              if (sectionNodeParent.getLocalName().startsWith("sect")) {
                insertionOffset = sectionNodeParent.getEndOffset() + 1;
              }
            }
          }
        }
      }
    } else {
      // Compute insertion offset for demote action.
      if (sectionNodeToProcess.getNamespace().isEmpty() || sectionNodeToProcess.getNamespace().equals(DOCBOOK5_NAMESPACE)) {
        if (sectionNodeToProcess.getLocalName().equals("section") || !nodeContainsSect5Element(sectionNodeToProcess)) {
          // Check if it has a previous section sibling.
          AuthorElementBaseInterface parentElement = sectionNodeToProcess.getParentElement();
          if (parentElement != null) {
            List<AuthorNode> childNodes = parentElement.getContentNodes();
            int sectionNodeToProcessIndex = childNodes.indexOf(sectionNodeToProcess);
            if (sectionNodeToProcessIndex != -1 && sectionNodeToProcessIndex > 0) {
              AuthorNode previousSibling = parentElement.getContentNodes().get(sectionNodeToProcessIndex - 1);
              if (previousSibling instanceof AuthorElement) {
                AuthorElement previousSiblingElement = (AuthorElement) previousSibling;
                if ((previousSiblingElement.getLocalName().equals("section")
                    || previousSiblingElement.getLocalName().startsWith("sect"))
                    && (previousSiblingElement.getNamespace().isEmpty() || previousSiblingElement.getNamespace().equals(DOCBOOK5_NAMESPACE))) {
                  insertionOffset = previousSibling.getEndOffset();
                }
              }
            }
          }
        }
      }
    }

    return insertionOffset;
  }
  
  /**
   * Changes the sect node names for a promote/demote action.
   * 
   * @param authorAccess The author access.
   * @param authorElement The current author element. 
   * @param action The promote/demote action.
   */
  private static void changeSectNodeName(AuthorAccess authorAccess, AuthorElement authorElement, PromoteDemote action) {
    if (authorElement != null) {
      if (authorElement.getLocalName().startsWith("sect") 
          && (authorElement.getNamespace().isEmpty() || (authorElement.getNamespace().equals(DOCBOOK5_NAMESPACE)))) {
        try {
          int currentIndex = Integer.parseInt(authorElement.getLocalName().substring("sect".length()));
          if (action == PromoteDemote.DEMOTE) {
            // Action is demote. Increment all sect indices.
            if (currentIndex < 5) {
              currentIndex++;
            }
          } else {
            // Action is promote. Decrement all sect indices.
            if (currentIndex > 1) {
              currentIndex--;
            }
          }
          String newSectNodeName = "sect" + currentIndex;
          String prefix = XmlUtil.getProxy(authorElement.getName());
          if (prefix != null && !prefix.isEmpty()) {
            newSectNodeName = prefix + ":" + newSectNodeName;
          }
          authorAccess.getDocumentController().renameElement(authorElement, newSectNodeName);
        } catch (NumberFormatException e) {
          // Do nothing
        }
      }
      List<AuthorNode> contentNodes = authorElement.getContentNodes();
      for (int i = 0; i < contentNodes.size(); i++) {
        if (contentNodes.get(i) instanceof AuthorElement) {
          changeSectNodeName(authorAccess, (AuthorElement)contentNodes.get(i), action);
        }
      }
    }
  }
  
  /**
   * Returns <code>true</code> if the sect node contains a "sect5" element.
   * 
   * @param sectionElement The sect node.
   * 
   * @return <code>true</code> if the sect node contains a "sect5" element.
   */
  public static boolean nodeContainsSect5Element(AuthorElement sectionElement) {
    boolean toReturn = false;
    
    if (sectionElement.getNamespace().isEmpty() || sectionElement.getNamespace().equals(DOCBOOK5_NAMESPACE)) {
      List<AuthorNode> contentNodes = sectionElement.getContentNodes();
      if (contentNodes.size() == 0) {
        // Is leaf.
        toReturn = sectionElement.getLocalName().equals("sect5");
      } else {
        if (!sectionElement.getLocalName().equals("sect5")) {
          for (int i = 0; i < contentNodes.size(); i++) {
            if (contentNodes.get(i) instanceof AuthorElement) {
              if (nodeContainsSect5Element((AuthorElement) contentNodes.get(i))) {
                toReturn = true;
                break;
              }
            }
          }
        } else {
          // Inner sect5 node.
          toReturn = true;
        }
      }
    }
    
    return toReturn;
  }
}
