/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.component.editor;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Icon;
import javax.swing.JMenuItem;
import javax.swing.JToggleButton;
import javax.swing.border.Border;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.extensions.api.AuthorExtensionActionProvider;
import ro.sync.ecss.extensions.api.CursorType;
import ro.sync.ecss.extensions.api.editor.AbstractInplaceEditor;
import ro.sync.ecss.extensions.api.editor.AuthorInplaceContext;
import ro.sync.ecss.extensions.api.editor.EditingEvent;
import ro.sync.ecss.extensions.api.editor.IAuthorExtensionAction;
import ro.sync.ecss.extensions.api.editor.InplaceEditorArgumentKeys;
import ro.sync.ecss.extensions.api.editor.InplaceRenderer;
import ro.sync.ecss.extensions.api.editor.RendererLayoutInfo;
import ro.sync.exml.EXMLResourceBoundle;
import ro.sync.exml.MessageBundle;
import ro.sync.exml.Tags;
import ro.sync.exml.view.graphics.Font;
import ro.sync.exml.view.graphics.Point;
import ro.sync.exml.view.graphics.Rectangle;
import ro.sync.ui.Icons;
import ro.sync.ui.ImageUtil;
import ro.sync.ui.application.ApplicationPopupMenu;
import ro.sync.ui.tooltip.MultiLineToolTip;
import ro.sync.util.PlatformDetector;

/**
 * Inplace editor that uses a button to trigger a pop-up menu with multiple actions. 
 * 
 * @author adriana_sbircea
 */
@API(type=APIType.INTERNAL, src=SourceType.PUBLIC)
public class ButtonGroupEditor extends AbstractInplaceEditor implements InplaceRenderer  {
  private static final MessageBundle messages = EXMLResourceBoundle.getResourceBundleInstance();

  /** 
   * The button that triggers the pop-up menu. 
   */
  final private JToggleButton button; 
  
  /** 
   * The pop-up menu that contains all the actions. 
   */
  final private ApplicationPopupMenu popupMenu;

  /** 
   * Action listener on the button. 
   */
  private ActionListener actionListener;
  
  /** 
   * Key listener on the button. 
   * */
  private KeyListener keyListener;
  
  /** 
   * Default font of the button and pop-up menu. 
   */
  final private java.awt.Font defaultFont;
  
  /** 
   * Default color of the button and pop-up menu. 
   */
  final private Color defaultColor;
  
  /**
   * The default border.
   */
  private Border defaultBorder;
  
  /**
   * Constructor.
   */
  public ButtonGroupEditor() {
    button = new JToggleButton() {
      @Override
      public javax.swing.JToolTip createToolTip() {
        MultiLineToolTip tooltip = new MultiLineToolTip();
        tooltip.setComponent(this);
        return tooltip;
      }

      @Override
      protected void paintComponent(Graphics g) {
        if ( 
            // Only if it's flat.
            !isContentAreaFilled() 
            && (getModel().isRollover() || isSelected())) {
          ButtonEditor.paintRolloverBox(g, this.getWidth(), this.getHeight());
        }
        
        super.paintComponent(g);
      }
    };
  
    defaultFont = button.getFont();
    defaultColor = button.getForeground();
    defaultBorder = button.getBorder();
    
    popupMenu = new ApplicationPopupMenu(null, null);
    
    // This ensures that the button is selected iff the popup menu is visible. 
    popupMenu.addPopupMenuListener(new PopupMenuListener() {
      @Override
      public void popupMenuWillBecomeVisible(PopupMenuEvent e) {
        button.setSelected(true);
      }
      @Override
      public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {
        button.setSelected(false);
      }
      @Override
      public void popupMenuCanceled(PopupMenuEvent e) {
        button.setSelected(false);
      }
    });

  }
  
  /**
   * @see ro.sync.ecss.extensions.api.Extension#getDescription()
   */
  @Override
  public String getDescription() {
    return "A button group that shows a popup to choose among several extension actions";
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#getEditorComponent(ro.sync.ecss.extensions.api.editor.AuthorInplaceContext, ro.sync.exml.view.graphics.Rectangle, ro.sync.exml.view.graphics.Point)
   */
  @Override
  public Object getEditorComponent(AuthorInplaceContext context, Rectangle allocation,
      Point mouseInvocationLocation) {
    prepare(context);
    
    return button;
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#getScrollRectangle()
   */
  @Override
  public Rectangle getScrollRectangle() {
    return null;
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#requestFocus()
   */
  @Override
  public void requestFocus() {
    button.requestFocus();
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#getValue()
   */
  @Override
  public Object getValue() {
    return null;
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#stopEditing()
   */
  @Override
  public void stopEditing() {
    popupMenu.setVisible(false);
    fireEditingStopped(new EditingEvent((String) null));
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#cancelEditing()
   */
  @Override
  public void cancelEditing() {
    popupMenu.setVisible(false);
    fireEditingCanceled();
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceRenderer#getRendererComponent(ro.sync.ecss.extensions.api.editor.AuthorInplaceContext)
   */
  @Override
  public Object getRendererComponent(AuthorInplaceContext context) {
    prepare(context);

    return button;
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceRenderer#getCursorType(ro.sync.ecss.extensions.api.editor.AuthorInplaceContext, int, int)
   */
  @Override
  public CursorType getCursorType(AuthorInplaceContext context, int x, int y) {
    return null;
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceRenderer#getCursorType(int, int)
   */
  @Override
  public CursorType getCursorType(int x, int y) {
    return CursorType.CURSOR_NORMAL;
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceRenderer#getTooltipText(ro.sync.ecss.extensions.api.editor.AuthorInplaceContext, int, int)
   */
  @Override
  public String getTooltipText(AuthorInplaceContext context, int x, int y) {
    return getTooltipText(context);
  }

  /** 
   * @return the font for the button and pop-up menu that is specified in the context. If no font 
   * is specified, a default one is returned. 
   */
  private java.awt.Font getFont(AuthorInplaceContext context) {
    Font font = (Font) context.getArguments().get(InplaceEditorArgumentKeys.FONT);
    if (font != null) {
      return new java.awt.Font(font.getName(), font.getStyle(), font.getSize());
    } 
    return defaultFont;
  }

  /** 
   * @return the foreground color of the button and pop-up menu that is specified in the context.
   * If no color is specified, a default one is returned. 
   */
  private Color getForegroundColor(AuthorInplaceContext context) {
    ro.sync.exml.view.graphics.Color color = (ro.sync.exml.view.graphics.Color) context
        .getArguments().get(InplaceEditorArgumentKeys.PROPERTY_COLOR);
    if (color != null) {
      return new Color(color.getRed(), color.getGreen(), color.getBlue(), color.getAlpha());
    }
    return defaultColor;
  }

  /**
   * Loads an icon with the given path.
   * 
   * @param iconPath the path specified in the CSS for the icon.
   * @return the loaded icon.
   * @throws FileNotFoundException if the icon cannot be found.
   */
  private Icon loadIcon(String iconPath) throws FileNotFoundException {
    Icon icon;
    if ((icon = ImageUtil.loadMenuIcon(iconPath)) == null) {
      if ((icon = ImageUtil.loadToolbarIcon(iconPath)) == null) {
        throw new FileNotFoundException("File " + iconPath + " not found.");
      }
    }
    return icon;
  }
  
  /**
   * Loads the icon specified in the context. 
   * 
   * @param context the context of the in-place editor.
   * @return the icon, or a default one if there was no icon specified or there was an error while 
   * loading it.
   */
  private Icon getIcon(AuthorInplaceContext context) {
    Object iconProp = context.getArguments().get(InplaceEditorArgumentKeys.PROPERTY_ICON);
    if (iconProp instanceof String) {
      Icon icon;
      try {
        icon = loadIcon((String) iconProp);
      } catch (FileNotFoundException e) {
        icon = Icons.getIcon(Icons.MISSING_RESOURCE);
      }
      context.getArguments().put(InplaceEditorArgumentKeys.PROPERTY_ICON, icon);
      return icon;
    } else {
      return (Icon) iconProp;
    }
  }
  
  /** 
   * Returns the text of the button. 
   * 
   * @param context the context of the editor.
   */
  private String getText(AuthorInplaceContext context) {
    return (String) context.getArguments().get(InplaceEditorArgumentKeys.PROPERTY_LABEL); 
  }
  
  /** 
   * Returns the tooltip text for the button. 
   * 
   * @param the context of the editor.
   */
  private String getTooltipText(AuthorInplaceContext context) {
    return (String) context.getArguments().get(InplaceEditorArgumentKeys.PROPERTY_TOOLTIP);
  }
  
  /**
   * Resolves the IDs of the actions to be displayed in the pop-up menus.
   * 
   * @param actionProvider action provider used to resolve the IDs of the actions.
   * @param actionIds the comma separated IDs of the actions.
   * @return a list of IAuthorExtensionActions
   */
  public static List<IAuthorExtensionAction> processActions(
      AuthorExtensionActionProvider actionProvider, String actionIds) {
    List<IAuthorExtensionAction> actions = new ArrayList<IAuthorExtensionAction>();
    for (String actionId: actionIds.split(",")) {
      actionId = actionId.trim();
      IAuthorExtensionAction extensionAction = null;
      if (actionProvider != null) {
        extensionAction = actionProvider.getExtensionAction(actionId); 
      }
      if (extensionAction != null) {
        actions.add(extensionAction);
      } else {
        throw new IllegalArgumentException("Invalid extension action ID: \"" + actionId +"\".");
      }
    }
    if (actions.isEmpty()) {
      throw new IllegalArgumentException("No action was specified.");
    }
    return actions;
  }
  
  /**
   * @param context the context of the editor.
   * @return the actions to be included in the pop-up menu.
   */
  private List<IAuthorExtensionAction> getActions(AuthorInplaceContext context) {
    List<IAuthorExtensionAction> actions = (List<IAuthorExtensionAction>) context.getArguments()
        .get(InplaceEditorArgumentKeys.PROPERTY_ACTION_IDS);
    if (actions != null) {
      return actions;
    }
    return Collections.emptyList();
  }

  /** 
   *  Display style for the action in the pop-up menu.
   * 
   *  It can be text-only, icon-only or both.
   */
  public static enum ActionDisplayStyle {
    /** Display only text for the menu items. */
    TEXT,
    /** Display only icon for the menu items. */
    ICON,
    /** Display both icon and text for the menu items. */
    BOTH
  }
  
  /**
   * Processes the action display style. Makes sure that it is one of the three values: 
   * text, icon, both.
   * 
   * @param actionDisplayStyle the style specified in CSS.
   * @return the style as enum.
   */
  public static ActionDisplayStyle processActionDisplayStyle(String actionDisplayStyle) {
    try {
      if (actionDisplayStyle.equals(actionDisplayStyle.toLowerCase())) {
        return ActionDisplayStyle.valueOf(actionDisplayStyle.toUpperCase());
      }
    } catch (IllegalArgumentException ex) {
    }
    throw new IllegalArgumentException(
        "Invalid actionDisplayStyle value. Valid ones are: text, icon, both.");
  }
  
  /**
   * Retrieves the action display style from the context.
   * 
   * @param context the context.
   * @return the display style of the actions or TEXT by default.
   */
  private ActionDisplayStyle getActionDisplayStyle(AuthorInplaceContext context) {
    ActionDisplayStyle style = (ActionDisplayStyle) context.getArguments()
        .get(InplaceEditorArgumentKeys.PROPERTY_ACTION_DISPLAY_STYLE);
    return style != null ? style : ActionDisplayStyle.TEXT;
  }
  
  /**
   * Prepares the editor for the given context.
   * 
   * @param context the context.
   */
  private void prepare(final AuthorInplaceContext context) {
    button.setFont(getFont(context));
    button.setForeground(getForegroundColor(context));

    Icon icon = getIcon(context);
    button.setIcon(icon);

    // If an icon is specified, it is used, otherwise the specified text or a default one.
    String text = getText(context);
    if (icon != null) {
      button.setText(null);
    } else if (text != null){
      button.setText(text);
    } else {
      button.setText(messages.getString(Tags.UNKNOWN));
    }
    button.setToolTipText(getTooltipText(context));
    
    // Offers the button in the visual appearance the ButtonEditor has. 
    ButtonEditor.decorateJButton(button, context, defaultBorder);

    if (actionListener != null) {
      button.removeActionListener(actionListener);
    }
    if (keyListener != null) {
      button.removeKeyListener(keyListener);
    }
    
    actionListener = new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent e) {
        showPopup(context);
      }
    };
    button.addActionListener(actionListener);
    
    keyListener = new KeyAdapter() {
      @Override
      public void keyPressed(KeyEvent e) {
        if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
          // End the editing session.
          e.consume();
          fireEditingCanceled();
        } else if (e.getKeyCode() == KeyEvent.VK_ENTER) {
          showPopup(context);
        }
      }
    };
    button.addKeyListener(keyListener);
  }
  
  /** 
   *  Shows the pop-up with the actions that correspond to the given context.
   * 
   *  @param context the context.
   */
  private void showPopup(AuthorInplaceContext context) {
    popupMenu.setFont(getFont(context));
    popupMenu.setForeground(getForegroundColor(context));
    
    popupMenu.removeAll();
    ActionDisplayStyle style = getActionDisplayStyle(context);
    if (style != ActionDisplayStyle.ICON) {
      for (IAuthorExtensionAction action : getActions(context)) {
        popupMenu.add(new WrappingAction(action, style));
      }
    } else {
      // The pop-up menu leaves a blank space for the text of the action even if it 
      // is not specified. We resize the menu to hide that space. 
      Insets margin = new Insets(2,2,2,2);
      int maxItemWidth = 0;
      int height = 0;
      
      for (IAuthorExtensionAction action : getActions(context)) {
        WrappingAction wrappingAction = new WrappingAction(action, style);
        if (style == ActionDisplayStyle.ICON) {
          JMenuItem menuItem = new JMenuItem(wrappingAction);
          popupMenu.add(menuItem);
          // No text, no space between Icon and text and a margin of 2 pixels.
          menuItem.setHideActionText(true);
          menuItem.setIconTextGap(0);
          menuItem.setMargin(margin);

          Insets border = menuItem.getBorder().getBorderInsets(menuItem);
          // Compute the width of the icon part of the MenuItem.    
          int width = border.left + margin.left + menuItem.getIcon().getIconWidth() + margin.right + border.right; 
          if (PlatformDetector.isMacOS()) {
            // Hack: On Mac, the icon is shifted to the right by 10px (besides 
            // the margin and the border which are accessible through API).
            width += 20;
          }
          maxItemWidth = Math.max(maxItemWidth, width);
          // Compute the height of the MenuItem.
          height += border.top + margin.top + menuItem.getIcon().getIconHeight() + margin.bottom + border.bottom;
        } else {
          popupMenu.add(wrappingAction);
        }
      }
      
      Insets popupMargin = popupMenu.getMargin();
      Insets popupBorder = popupMenu.getBorder().getBorderInsets(popupMenu);

      int popupWidth = popupBorder.left + popupMargin.left + maxItemWidth + popupMargin.right + popupBorder.right;
      int popupHeight = popupBorder.top + popupMargin.top + height + popupMargin.bottom + popupBorder.bottom;
      popupMenu.setPreferredSize(new Dimension(popupWidth, popupHeight));
    }

    popupMenu.show(button, 0, button.getBounds().height);
  }
  
  /** 
   * Gets the icon of an author extension action according to a given style.
   * 
   * @param authorAction the extension action
   * @param style the style.
   */
  private static Icon getIcon(IAuthorExtensionAction authorAction, ActionDisplayStyle style) {
    switch (style) {
      case ICON:
      case BOTH:
        String iconPath = (String) authorAction.getValue(IAuthorExtensionAction.SMALL_ICON_PATH);
        Icon icon = ImageUtil.loadMenuIcon(iconPath);
        if (icon == null && style == ActionDisplayStyle.ICON) {
          // If we really need to display an icon, use a default one.
          icon = Icons.getIcon(Icons.MISSING_RESOURCE);
        }
        return icon;
      default:
        // Should not reach this.
        return null;
    } 
  }
  
  /** 
   * Gets the name of an author extension action according to a given style.
   * 
   * @param authorAction the extension action
   * @param style the style.
   */
  private static String getName(IAuthorExtensionAction authorAction, ActionDisplayStyle style) {
    switch (style) {
      case TEXT:
      case BOTH:
        String name = (String) authorAction.getValue(IAuthorExtensionAction.ACTION_NAME);
        assert (name != null);
        return name;
      default:
        return null;
    }
  }

  /** 
   * A swing action that stops the editing session by delegating to the given autorAction. 
   */
  private class WrappingAction extends AbstractAction {
    /**
     * The wrapped author action.
     */
    private IAuthorExtensionAction authorAction;
    
    /**
     * Constructor.
     * 
     * @param authorAction the action that we delegate to. 
     * @param style the style used to display the Action (icon, text or both).
     */
    public WrappingAction(IAuthorExtensionAction authorAction, ActionDisplayStyle style) {
      super(getName(authorAction, style), getIcon(authorAction, style));
      this.putValue(AbstractAction.SHORT_DESCRIPTION, 
          authorAction.getValue(IAuthorExtensionAction.DESCRIPTION));
      this.authorAction = authorAction;
    }
    
    /**
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    @Override
    public void actionPerformed(ActionEvent e) {
      fireEditingStopped(new EditingEvent(authorAction));
      popupMenu.setVisible(false);
    }
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceRenderer#getRenderingInfo(ro.sync.ecss.extensions.api.editor.AuthorInplaceContext)
   */
  @Override
  public RendererLayoutInfo getRenderingInfo(AuthorInplaceContext context) {
    prepare(context);
    
    return ButtonEditor.getButtonRenderingInfo(button, context);
  }

}
