/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.component.editor;

import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.LayoutManager;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JToolTip;

import org.apache.log4j.Logger;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.contentcompletion.xml.CIValue;
import ro.sync.ecss.extensions.api.CursorType;
import ro.sync.ecss.extensions.api.editor.AbstractInplaceEditor;
import ro.sync.ecss.extensions.api.editor.AuthorInplaceContext;
import ro.sync.ecss.extensions.api.editor.EditingEvent;
import ro.sync.ecss.extensions.api.editor.InplaceEditorArgumentKeys;
import ro.sync.ecss.extensions.api.editor.InplaceEditorCSSConstants;
import ro.sync.ecss.extensions.api.editor.InplaceRenderer;
import ro.sync.ecss.extensions.api.editor.RendererLayoutInfo;
import ro.sync.exml.view.graphics.Dimension;
import ro.sync.exml.view.graphics.Font;
import ro.sync.exml.view.graphics.Point;
import ro.sync.exml.view.graphics.Rectangle;
import ro.sync.ui.application.ApplicationBorderFactory;
import ro.sync.ui.tooltip.MultiLineToolTip;

/**
 * A panel with checkboxes that can be used to render boolean values but also lists
 * of values.
 * 
 * @author alex_jitianu
 */
@API(type=APIType.INTERNAL, src=SourceType.PUBLIC)
public class CheckBoxEditor extends AbstractInplaceEditor implements InplaceRenderer {
  
  /**
   * Logger for logging.
   */
  private static final Logger logger = Logger.getLogger(CheckBoxEditor.class.getName());
  
  /**
   * The panel with the check-boxes.
   */
  private JPanel panel;
  /**
   * The checkboxes added inside the panel.
   */
  private List<JCheckBox> checks;
  /**
   * The values of property {@link InplaceEditorCSSConstants#PROPERTY_VALUES}.
   */
  private List<CIValue> values;
  /**
   * The values of property {@link InplaceEditorCSSConstants#PROPERTY_UNCHECKED_VALUES}.
   */
  private List<CIValue> uncheckedValues;
  /**
   * The value of property {@link InplaceEditorCSSConstants#PROPERTY_SEPARATOR}.
   */
  private String separator;
  
  /**
   * <code>true</code> if we are during a mouse event.
   */
  private boolean duringMouseEvent = false;
  
  /**
   * Prepare the renderer.
   * 
   * @param context In-place editing information.
   */
  private void prepare(AuthorInplaceContext context, boolean isRenderer) {
    boolean isSA = true;
    if (context.getAuthorAccess() != null && context.getAuthorAccess().getWorkspaceAccess() != null) {
      isSA = context.getAuthorAccess().getWorkspaceAccess().isStandalone();
    }
    // Add key listener on ESC to cancel the editing.
    KeyListener keyListener = null;
    FocusListener focusListener = null;
    ActionListener actionListener = null;
    if (!isRenderer) {
      keyListener = new KeyAdapter() {
        @Override
        public void keyPressed(KeyEvent e) {
          if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
            e.consume();
            cancelEditing();
          }
        }
      };

      // Add focus listener to stop editing when focus lost
      focusListener = new FocusAdapter() {
        @Override
        public void focusLost(FocusEvent e) {
          if (!checks.contains(e.getOppositeComponent())) {
            stopEditing(e.getOppositeComponent() == null);
          }
        }
      };

      actionListener = new ActionListener() {
        @Override
        public void actionPerformed(ActionEvent e) {
          fireEditingOccured();
          if (duringMouseEvent) {
            stopEditing();
          } else {
            // EXM-25433 Update the document.
            fireCommitValue(new EditingEvent((String) getValue()));
          }
        }
      };
    }
    
    // Compute the background color.
    ro.sync.exml.view.graphics.Color color =
        (ro.sync.exml.view.graphics.Color) context.getArguments().get(InplaceEditorArgumentKeys.BG_COLOR);
    Color awtColor = null;
    if (color != null) {
      awtColor = new Color(color.getRed(), color.getGreen(), color.getBlue(), color.getAlpha());
    }
    
    // Create the panel that holds the check box.
    panel = new JPanel();
    if (!isSA && awtColor != null) {
      panel.setBackground(awtColor);
      panel.setOpaque(true);
    } else {
      panel.setOpaque(false);
    }
    
    // Set an empty border to the panel that holds the check boxes.
    panel.setBorder(ApplicationBorderFactory.createEmptyBorder());
    checks = new ArrayList<JCheckBox>();

    // Get values, unchecked values and labels.
    values = (List<CIValue>) context.getArguments().get(InplaceEditorArgumentKeys.PROPERTY_VALUES);
    uncheckedValues = (List<CIValue>) context.getArguments().get(InplaceEditorArgumentKeys.PROPERTY_UNCHECKED_VALUES);
    List<String> labels = (List<String>) context.getArguments().get(InplaceEditorArgumentKeys.PROPERTY_LABELS);
    separator = (String) context.getArguments().get(InplaceEditorArgumentKeys.PROPERTY_SEPARATOR);
    // If no separator is specified, SPACE is used.
    if (separator == null) {
      separator = " ";
    }

    String initialValue = (String) context.getArguments().get(InplaceEditorArgumentKeys.INITIAL_VALUE);
    if (logger.isDebugEnabled()) {
      logger.debug("initialValue " + initialValue);
    }
    // Get the initial values
    List<String> initialValues = new ArrayList<String>();
    if (initialValue != null) {
      StringTokenizer stringTokenizer = new StringTokenizer(initialValue, separator);
      while (stringTokenizer.hasMoreTokens()) {
        String token = stringTokenizer.nextToken();
        initialValues.add(token.trim());
      }
    }
    
    String defaultValue = (String) context.getArguments().get(InplaceEditorArgumentKeys.DEFAULT_VALUE);
    if (logger.isDebugEnabled()) {
      logger.debug("defaultValue " + defaultValue);
    }
    
    Font font = (Font) context.getArguments().get(InplaceEditorArgumentKeys.FONT);
    color = (ro.sync.exml.view.graphics.Color) context.getArguments().get(InplaceEditorArgumentKeys.PROPERTY_COLOR);
    Color fAWTColor = null;
    if (color != null) {
      fAWTColor = new Color(color.getRed(), color.getGreen(), color.getBlue(), color.getAlpha());
    }
    
    // Get the default values
    List<String> defaultValues = new ArrayList<String>();
    if (defaultValue != null) {
      StringTokenizer stringTokenizer = new StringTokenizer(defaultValue, separator);
      while (stringTokenizer.hasMoreTokens()) {
        String token = stringTokenizer.nextToken();
        defaultValues.add(token.trim());
      }
    }
    
    LayoutManager layout = new FlowLayout(FlowLayout.LEFT, 0, 0);
    Integer columns = (Integer) context.getArguments().get(InplaceEditorArgumentKeys.PROPERTY_COLUMNS);
    if (columns != null && columns > 0) {
      layout = new java.awt.GridLayout(0, Math.min(columns, values.size()));
    }
    panel.setLayout(layout);
    
    // Create checkboxes for all values.
    for (int i = 0; i < values.size(); i++) {
      CIValue ciValue = values.get(i);
      String insertString = ciValue.getInsertString();
      String text = insertString;
      if (labels != null && !labels.isEmpty()) {
        // We have a label to present.
        text = labels.get(i);
      }

      JCheckBox checkBox = new JCheckBox(text) {
        @Override
        protected void processMouseEvent(MouseEvent e) {
          try {
            duringMouseEvent = true;
            super.processMouseEvent(e);
          } finally {
            duringMouseEvent = false;
          }
        }
        
        @Override
        public JToolTip createToolTip() {
          MultiLineToolTip tooltip = new MultiLineToolTip();
          tooltip.setComponent(this);
          return tooltip;
        }
      };
      
      checkBox.setToolTipText(ciValue.getAnnotation());
      
      if (font != null) {
        checkBox.setFont(new java.awt.Font(font.getName(), font.getStyle(), font.getSize()));
      }
      
      if (initialValues.contains(insertString)
          || (initialValue == null && defaultValues.contains(insertString))) {
        checkBox.setSelected(true);
      }
      
      if (!isSA && awtColor != null) {
        checkBox.setBackground(awtColor);
        checkBox.setOpaque(true);
      } else {
        checkBox.setOpaque(false);
      }
      
      if (fAWTColor != null) {
        checkBox.setForeground(fAWTColor);
      }
      
      if (!isRenderer) {
        checkBox.addKeyListener(keyListener);
        checkBox.addFocusListener(focusListener);
        checkBox.addActionListener(actionListener);
      }
      
      panel.add(checkBox);
      checks.add(checkBox);
    }
    
    if (isRenderer && initialValue == null && defaultValue != null) {
      // If we are presenting the default combination we'll mark it differently.
      for (JCheckBox checkBox : checks) {
        int style = Font.ITALIC;
        if (checkBox.getFont().getStyle() != Font.PLAIN) {
          style = style | checkBox.getFont().getStyle();
        }
        checkBox.setFont(checkBox.getFont().deriveFont(style));
      }
    }
    
    if (isRenderer && !checks.isEmpty()) {
      // If it's a renderer we'll need a character for the baseline to compute correctly.
      // Also the paint must have the space to paint where it should.
      JCheckBox firstCheck = checks.get(0);
      if (firstCheck.getText().length() == 0) {
        // No baseline is reported unless some text exists.
        firstCheck.setText(" ");
      }
    }
    
    // Check if we should highlight any of the buttons.
    Point point = context.getRelativeMouseLocation();
    if (isRenderer && point != null) {
      // We wont be able to locate the component under the mouse until the layout is computed.
      java.awt.Dimension size = panel.getPreferredSize();
      panel.setSize(size.width, size.height);
      panel.doLayout();
      
      Component componentAt = panel.getComponentAt(point.x, point.y);
      if (componentAt instanceof JCheckBox) {
        ((JCheckBox) componentAt).getModel().setRollover(true);
      }
    }
  }

  /**
   * @see ro.sync.ecss.extensions.api.Extension#getDescription()
   */
  @Override
  public String getDescription() {
    return "A renderer that uses checkboxes.";
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceRenderer#getRendererComponent(ro.sync.ecss.extensions.api.editor.AuthorInplaceContext)
   */
  @Override
  public Object getRendererComponent(AuthorInplaceContext context) {
    prepare(context, true);
    return panel;
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceRenderer#getRenderingInfo(ro.sync.ecss.extensions.api.editor.AuthorInplaceContext)
   */
  @Override
  public RendererLayoutInfo getRenderingInfo(AuthorInplaceContext context) {
    prepare(context, true);
    
    // Get the baseline
    int baseline = -1;
    if (!checks.isEmpty()) {
      JCheckBox firstCheck = checks.get(0);
      
      java.awt.Dimension preferredSize = firstCheck.getPreferredSize();
      baseline = firstCheck.getBaseline(preferredSize.width, preferredSize.height);
    }
    
    // Get the size
    java.awt.Dimension preferredSize = panel.getPreferredSize();
    
    return new RendererLayoutInfo(baseline, new Dimension(preferredSize.width, preferredSize.height));
  }

  //////////////// EDITOR METHODS ///////////////////
  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#getEditorComponent(ro.sync.ecss.extensions.api.editor.AuthorInplaceContext, ro.sync.exml.view.graphics.Rectangle, ro.sync.exml.view.graphics.Point)
   */
  @Override
  public Object getEditorComponent(AuthorInplaceContext context, Rectangle allocation, Point mouseLocation) {
    prepare(context, false);
    return panel;
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#requestFocus()
   */
  @Override
  public void requestFocus() {
    if (!checks.isEmpty()) {
      checks.get(0).requestFocus();
    }
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#getValue()
   */
  @Override
  public Object getValue() {
    StringBuilder valuesStr = new StringBuilder();
    
    for (int i = 0; i < checks.size(); i++) {
      String value = null;
      JCheckBox checkBox = checks.get(i);
      if (checkBox.isSelected()) {
        value = values.get(i).getInsertString();
      } else if (uncheckedValues != null && !uncheckedValues.isEmpty()) {
        // We have values that must be committed when the checkbox is unchecked.
        value = uncheckedValues.get(i).getInsertString();
      }
      
      if (value != null) {
        if (valuesStr.length() > 0) {
          valuesStr.append(separator);
        }
        
        valuesStr.append(value);
      }
    }
    
    return valuesStr.toString();
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#stopEditing()
   */
  @Override
  public void stopEditing() {
    stopEditing(true);
  }
  
  /**
   * Stops editing.
   * 
   * @param requestFocus <code>true</code> to request focus inside the author component.
   */
  private void stopEditing(boolean requestFocus) {
    fireEditingStopped(new EditingEvent((String) getValue(), requestFocus));
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#cancelEditing()
   */
  @Override
  public void cancelEditing() {
    fireEditingCanceled();
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#getScrollRectangle()
   */
  @Override
  public Rectangle getScrollRectangle() {
    Rectangle r = null;
    for (int i = 0; i < checks.size(); i++) {
      JCheckBox jCheckBox = checks.get(i);
      if (jCheckBox.isFocusOwner()) {
        java.awt.Rectangle bounds = jCheckBox.getBounds();
        r = new Rectangle(bounds.x, bounds.y, bounds.width, bounds.height);
        break;
      }
    }
    
    return r;
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceRenderer#getTooltipText(ro.sync.ecss.extensions.api.editor.AuthorInplaceContext, int, int)
   */
  @Override
  public String getTooltipText(AuthorInplaceContext context, int x, int y) {
    prepare(context, true);
    java.awt.Dimension size = panel.getPreferredSize();
    // Apply a size to compute the locations for the child components.
    panel.setSize(size.width, size.height);
    panel.doLayout();
    String tooltip = null;
    Component component = panel.getComponentAt(x, y);
    if (component instanceof JCheckBox) {
      int indexOf = checks.indexOf(component);
      CIValue ciValue = values.get(indexOf);
      tooltip = ciValue.getAnnotation();
    }
    return tooltip;
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceRenderer#getCursorType(ro.sync.ecss.extensions.api.editor.AuthorInplaceContext, int, int)
   */
  @Override
  public CursorType getCursorType(AuthorInplaceContext context, int x, int y) {
    return CursorType.CURSOR_NORMAL;
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceRenderer#getCursorType(int, int)
   */
  @Override
  public CursorType getCursorType(int x, int y) {
    return null;
  }
}