/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.component.editor;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JToolTip;
import javax.swing.ListSelectionModel;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.apache.log4j.Logger;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.contentcompletion.xml.CIValue;
import ro.sync.ecss.extensions.api.editor.AbstractInplaceEditor;
import ro.sync.ecss.extensions.api.editor.AuthorInplaceContext;
import ro.sync.ecss.extensions.api.editor.EditingEvent;
import ro.sync.ecss.extensions.api.editor.InplaceEditorArgumentKeys;
import ro.sync.ecss.extensions.api.editor.InplaceEditorCSSConstants;
import ro.sync.exml.view.graphics.Font;
import ro.sync.exml.view.graphics.Point;
import ro.sync.exml.view.graphics.Rectangle;
import ro.sync.ui.ListFocusTraversalPolicy;
import ro.sync.ui.application.ApplicationBorderFactory;
import ro.sync.ui.application.ApplicationListCellRendererWrapper;
import ro.sync.ui.application.ApplicationScrollPane;
import ro.sync.ui.tooltip.MultiLineToolTip;

/**
 * A panel with check-boxes that can be used to render boolean values but also lists
 * of values.
 * 
 * @author alex_jitianu
 */
@API(type=APIType.INTERNAL, src=SourceType.PUBLIC)
public class PopupListEditor extends AbstractInplaceEditor {
  
  /**
   * Logger for logging.
   */
  private static final Logger logger = Logger.getLogger(PopupListEditor.class.getName());

  /**
   * The size of the empty border surrounding the list. 
   */
  private static final int BORDER_SIZE = 5;
  /**
   * The panel with the check-boxes.
   */
  private final JList list;
  /**
   * The values of property {@link InplaceEditorCSSConstants#PROPERTY_VALUES}.
   */
  private List<CIValue> values;
  /**
   * If there is a number of rows imposed we will use a scroll pane.
   */
  private ApplicationScrollPane scrollPane;
  /**
   * Key listener to stop editing.
   */
  private KeyAdapter keyListener;
  /**
   * Focus listener to stop editing.
   */
  private FocusAdapter focusListener;
  /**
   * Selection listener to fire edit notification.
   */
  private ListSelectionListener actionListener;
  /**
   * Mouse listener to stop editing.
   */
  private MouseAdapter mouseListener;
  /**
   * The cell index at the mouse position in the list.
   */
  private int mousePositionIndex;

  /**
   * The default font for the list.
   */
  private java.awt.Font defaultFont;

  /**
   * The labels used for list items.
   */
  private List<String> labels;
  /**
   * The ascending values comparator.
   */
  private PopupCheckboxComparator ASCENDING_VALUES_COMPARATOR;
  
  /**
   * The descending values comparator.
   */
  private Comparator<CIValue> DESCENDING_VALUES_COMPARATOR;

  /**
   * Constructor.
   */
  public PopupListEditor() {
    ASCENDING_VALUES_COMPARATOR = new PopupCheckboxComparator();
    DESCENDING_VALUES_COMPARATOR = new Comparator<CIValue>() {
      @Override
      public int compare(CIValue o1, CIValue o2) {
        return - ASCENDING_VALUES_COMPARATOR.compare(o1, o2);
      }
    };
    
    list = new JList(new DefaultListModel()) {
      @Override
      protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        if (mousePositionIndex >= 0) {
          java.awt.Rectangle cellBounds = getCellBounds(mousePositionIndex, mousePositionIndex);
          Color color = UIManager.getColor("List.selectionBackground");
          if (color == null) {
            // Fall back on gray.
            color = Color.GRAY;
          }
          /*
           * Draw a very light highlight, filling exactly the space which would be filled by the
           * selection. In this way, we are sure at 1px precision that clicking anywhere in the
           * highlighted area, the corresponding row will be selected.
           */
          // Lighter filling, only inside area.
          g.setColor(new Color(color.getRed(), color.getGreen(), color.getBlue(), 32));
          // Fill only inside area
          g.fillRect(
              cellBounds.x + 1,
              cellBounds.y + 1,
              /*
               * -2 because inside area only
               * -1 to not paint margin over the column right vertical line (see drawRect() below).
               */
              cellBounds.width - 3,
              cellBounds.height - 2);
          // Darker border around filled area.
          g.setColor(new Color(color.getRed(), color.getGreen(), color.getBlue(), 64));
          g.drawRect(
              cellBounds.x,
              cellBounds.y,
              /*
               * -1 to keep line inside given bounds (rect painted at x+width, y+height which are outside)
               * -1 to not paint over the column right vertical line
               */
              cellBounds.width - 2,
              cellBounds.height - 1);
        }
      }
      
      @Override
      public JToolTip createToolTip() {
        MultiLineToolTip tooltip = new MultiLineToolTip();
        tooltip.setComponent(this);
        return tooltip;
      }
    };
    list.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    defaultFont = list.getFont();
    
    // Set renderer.
    list.setCellRenderer(new ApplicationListCellRendererWrapper(list.getCellRenderer()) {
    @Override
    public Component getListCellRendererComponent(JList list, Object value, int index,
        boolean isSelected, boolean cellHasFocus) {
      String text = ((CIValue) value).getValue();
      if (labels != null && !labels.isEmpty()) {
        // We have a label to present.
        text = labels.get(values.indexOf(value));
      }
      
      // EXM-28363 If the value is empty and there is no label specified, use a place holder
      if (text == null || text.length() == 0) {
        text = "[Empty]";
      }
      
      JComponent comp = (JComponent) super.getListCellRendererComponent(list, text, index, isSelected, cellHasFocus);
      comp.setToolTipText(((CIValue)value).getAnnotation());
      comp.setBorder(
          ApplicationBorderFactory.createCompoundBorder(
              comp.getBorder(),
              ApplicationBorderFactory.createEmptyBorder(0, BORDER_SIZE, 0, BORDER_SIZE)));
      return comp;
    }});
    
    // Add a key listener to cancel the editing when enter, escape or space is pressed.
    keyListener = new KeyAdapter() {
      @Override
      public void keyPressed(KeyEvent e) {
        if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
          // On ESC cancel the editing and do not commit the value.
          e.consume();
          cancelEditing();
        } else if (e.getKeyCode() == KeyEvent.VK_ENTER || e.getKeyCode() == KeyEvent.VK_SPACE) {
          // On ENTER or space cancel the editing and commit the value.
          e.consume();
          stopEditing(false, true);
        }
      }
    };
    
    focusListener = new FocusAdapter() {
      @Override
      public void focusLost(FocusEvent e) {
        stopEditing(e.getOppositeComponent() == null, false);
      }
      @Override
      public void focusGained(final FocusEvent e) {
        // We might have a scroll so lets make sure it's visible.
        list.ensureIndexIsVisible(list.getSelectedIndex());
      }
    };
    
    actionListener = new ListSelectionListener() {
      @Override
      public void valueChanged(ListSelectionEvent e) {
        if (!e.getValueIsAdjusting()) {
          fireEditingOccured();
        }
      }
    };
    
    mouseListener = new MouseAdapter() {
      @Override
      public void mouseClicked(MouseEvent e) {
        if (!e.isPopupTrigger()) {
          e.consume();
          stopEditing();
        }
      }
      @Override
      public void mouseMoved(MouseEvent e) {
        int oldIndex = mousePositionIndex;
        mousePositionIndex = list.locationToIndex(e.getPoint());
        
        if (mousePositionIndex != -1) {
          list.repaint(list.getCellBounds(mousePositionIndex, mousePositionIndex));
        }
        
        if (mousePositionIndex != oldIndex) {
          list.repaint(list.getCellBounds(oldIndex, oldIndex));
        }
      }
    };
  }
  
  /**
   * Prepare the renderer.
   * 
   * @param context In-place editing information.
   */
  private void prepare(AuthorInplaceContext context) {
    DefaultListModel model = (DefaultListModel) list.getModel();
    model.clear();
    
    values = (List<CIValue>) context.getArguments().get(InplaceEditorArgumentKeys.PROPERTY_VALUES);
    labels = (List<String>) context.getArguments().get(InplaceEditorArgumentKeys.PROPERTY_LABELS);

    final String initialValue = (String) context.getArguments().get(InplaceEditorArgumentKeys.INITIAL_VALUE);
    if (logger.isDebugEnabled()) {
      logger.debug("initialValue " + initialValue);
    }
    
    final String defaultValue = (String) context.getArguments().get(InplaceEditorArgumentKeys.DEFAULT_VALUE);
    if (logger.isDebugEnabled()) {
      logger.debug("defaultValue " + defaultValue);
    }
    
    Font font = (Font) context.getArguments().get(InplaceEditorArgumentKeys.FONT);
    if (font != null) {
      list.setFont(new java.awt.Font(font.getName(), java.awt.Font.PLAIN, font.getSize()));
      
    } else {
      list.setFont(defaultFont);
    }
    
    ro.sync.exml.view.graphics.Color color =
        (ro.sync.exml.view.graphics.Color) context.getArguments().get(InplaceEditorArgumentKeys.PROPERTY_COLOR);
    if (color != null) {
      list.setForeground(new Color(color.getRed(), color.getGreen(), color.getBlue(), color.getAlpha()));
    } 
    
    //Maybe we have to sort the rendered values.
    Object obj = context.getArguments().get(InplaceEditorArgumentKeys.PROPERTY_EDITOR_SORT);
    ASCENDING_VALUES_COMPARATOR.setValues(values);
    ASCENDING_VALUES_COMPARATOR.setLabels(labels);
    if(InplaceEditorArgumentKeys.SORT_ASCENDING.equals(obj)) {
      Collections.sort(values, ASCENDING_VALUES_COMPARATOR);
      Collections.sort(labels, PopupCheckBoxUtil.ASCENDING_LABELS_COMPARATOR);
    } else if(InplaceEditorArgumentKeys.SORT_DESCENDING.equals(obj)) {
      Collections.sort(values, DESCENDING_VALUES_COMPARATOR);
      Collections.sort(labels, PopupCheckBoxUtil.DESCENDING_LABELS_COMPARATOR);
    }
    
    CIValue toSelect = new CIValue(initialValue != null ? initialValue : defaultValue);
    // Create checkboxes for all values.
    for (int i = 0; i < values.size(); i++) {
      CIValue ciValue = values.get(i);
      
      model.addElement(ciValue);
      if (ciValue.getInsertString().equals(toSelect)) {
        toSelect = ciValue;
      }
    }
    
    // Add required listeners.
    list.addKeyListener(keyListener);
    list.addFocusListener(focusListener);
    list.getSelectionModel().addListSelectionListener(actionListener);
    list.addMouseListener(mouseListener);
    list.addMouseMotionListener(mouseListener);
    
    Border emptyBorder = ApplicationBorderFactory.createEmptyBorder(BORDER_SIZE, 0, BORDER_SIZE, 0);
    int rows = PopupCheckBoxUtil.getRowsProperty(context);
    if (rows > 0 && model.getSize() > rows) {
      // We need a scroll pane.
      scrollPane = new ApplicationScrollPane(
          list, 
          ApplicationScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
          ApplicationScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
      scrollPane.getVerticalScrollBar().setBlockIncrement(5);
      scrollPane.getVerticalScrollBar().setUnitIncrement(5);
      
      List<Component> comps = new ArrayList<Component>();
      comps.add(list);
      scrollPane.setFocusTraversalPolicy(new ListFocusTraversalPolicy(
          scrollPane, 
          comps, 
          comps.isEmpty() ? null : comps.get(0), false));
      scrollPane.setFocusTraversalPolicyProvider(true);
      
      list.setBorder(emptyBorder);
    } else {
      list.setBorder(
          ApplicationBorderFactory.createCompoundBorder(
              ApplicationBorderFactory.createLineBorder(Color.BLACK, 1),
              emptyBorder));
    }
    
    list.setSelectedValue(toSelect, true);
  }

  /**
   * @see ro.sync.ecss.extensions.api.Extension#getDescription()
   */
  @Override
  public String getDescription() {
    return "A renderer that uses checkboxes to allow multiple selection.";
  }

  //////////////// EDITOR METHODS ///////////////////
  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#getEditorComponent(ro.sync.ecss.extensions.api.editor.AuthorInplaceContext, ro.sync.exml.view.graphics.Rectangle, ro.sync.exml.view.graphics.Point)
   */
  @Override
  public Object getEditorComponent(AuthorInplaceContext context, Rectangle allocation, Point mouseLocation) {
    prepare(context);
    Dimension preferredSize = null;
    if (scrollPane != null && list.getModel().getSize() > 0) {
      // We have a scroll pane.
      final Component listCellRendererComponent =
          list.getCellRenderer().getListCellRendererComponent(
              list, list.getModel().getElementAt(0), 0, false, false);

      // Compute preferred size.
      Dimension checkSize = listCellRendererComponent.getPreferredSize();
      preferredSize = list.getPreferredSize();
      int rows = PopupCheckBoxUtil.getRowsProperty(context);
      preferredSize.height = checkSize.height * rows + 2 * BORDER_SIZE;
      preferredSize.width += scrollPane.getVerticalScrollBar().getPreferredSize().width;
    } else {
      preferredSize = list.getPreferredSize();
    }
    // We want to present the editor relative to the mouse location.
    if (mouseLocation != null) {
      allocation.x = mouseLocation.x;
    }
    
    // Update bounds
    allocation.y += allocation.height;
    allocation.width = preferredSize.width;
    allocation.height = preferredSize.height;
    
    return scrollPane != null ? scrollPane : list;
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#requestFocus()
   */
  @Override
  public void requestFocus() {
    list.requestFocus();
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#getValue()
   */
  @Override
  public Object getValue() {
    String value = null;
    if (list.getSelectedIndex() != -1) {
      value = ((CIValue)list.getSelectedValue()).getInsertString();
    }
    
    return value;
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#stopEditing()
   */
  @Override
  public void stopEditing() {
    stopEditing(true, false);
  }
  
  /**
   * Stops editing.
   * 
   * @param requestFocus <code>true</code> to request focus inside the author component.
   */
  private void stopEditing(boolean requestFocus, boolean onEnter) {
    list.removeKeyListener(keyListener);
    list.removeFocusListener(focusListener);
    list.getSelectionModel().removeListSelectionListener(actionListener);
    list.removeMouseListener(mouseListener);
    list.removeMouseMotionListener(mouseListener);
    
    if (onEnter) {
      fireNextEditLocationRequested();
    } else {
      fireEditingStopped(new EditingEvent((String) getValue(), requestFocus));
    }
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#cancelEditing()
   */
  @Override
  public void cancelEditing() {
    list.removeKeyListener(keyListener);
    list.removeFocusListener(focusListener);
    list.getSelectionModel().removeListSelectionListener(actionListener);
    list.removeMouseListener(mouseListener);
    list.removeMouseMotionListener(mouseListener);
    
    fireEditingCanceled();
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#getScrollRectangle()
   */
  @Override
  public Rectangle getScrollRectangle() {
    return null;
  }
}