/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.component.editor;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.BadLocationException;

import org.apache.log4j.Logger;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.contentcompletion.editor.CCTextField;
import ro.sync.ecss.extensions.api.CursorType;
import ro.sync.ecss.extensions.api.editor.AbstractInplaceEditor;
import ro.sync.ecss.extensions.api.editor.AuthorInplaceContext;
import ro.sync.ecss.extensions.api.editor.EditingEvent;
import ro.sync.ecss.extensions.api.editor.InplaceEditorArgumentKeys;
import ro.sync.ecss.extensions.api.editor.InplaceRenderer;
import ro.sync.ecss.extensions.api.editor.RendererLayoutInfo;
import ro.sync.ecss.extensions.commons.editor.InplaceEditorUtil;
import ro.sync.exml.view.graphics.Font;
import ro.sync.exml.view.graphics.Point;
import ro.sync.exml.view.graphics.Rectangle;
import ro.sync.ui.UndoableTextField;
import ro.sync.util.PlatformDetector;

/**
 * A text field with CC support.
 * 
 * @author alex_jitianu
 */
@API(type=APIType.INTERNAL, src=SourceType.PUBLIC)
public class TextFieldEditor extends AbstractInplaceEditor implements InplaceRenderer, LoggableInplaceRenderer {
  
  /**
   * Logger for logging.
   */
  private static final Logger logger = Logger.getLogger(TextFieldEditor.class.getName());

  /**
   * A text field with CC support.
   */
  private UndoableTextField textField = null;
  /**
   * Default color.
   */
  private Color defaultForeground;
  /**
   * The CC manager. We can avoid re-creating the manager any time we prepare 
   * the editor. We just change the context it uses.
   */
  private EditorCCManager ccManager;
  /**
   * The focus listener to stop editing.
   */
  private FocusAdapter focusListener;
  /**
   * Key listener to stop editing.
   */
  private KeyAdapter keyListener;
  /**
   * Imposed insets.
   */
  private Insets imposedInsets;

  /**
   * The default font.
   */
  private java.awt.Font defaultFont;

  /**
   * Constructor.
   */
  public TextFieldEditor() {
    // No point in creating so many managers.
    ccManager = new EditorCCManager();
    
    // Add focus listener to commit the the selected value on focus lost
    focusListener = new FocusAdapter() {
      @Override
      public void focusLost(FocusEvent e) {
        if (logger.isDebugEnabled()) {
          logger.debug("Focus lost " + e.getOppositeComponent());
        }
        // Commit but request focus only if there is no focus destination.
        stopEditing(e.getOppositeComponent() == null, false);
      }
    };

    keyListener = new KeyAdapter() {
      @Override
      public void keyPressed(KeyEvent e) {
        if (e.getKeyCode() == KeyEvent.VK_ESCAPE 
            && !((CCTextField) textField).getCCSupport().getContentCompletionWindow().isVisible()) {
          // An Escape while the popUp is not visible must cancel the edit.
          e.consume();
          cancelEditing();
        } else if (e.getKeyCode() == KeyEvent.VK_ENTER 
            && !((CCTextField) textField).getCCSupport().getContentCompletionWindow().isVisible()) {
          // An ENTER commits the value.
          e.consume();
          stopEditing(false, true);
        }
      }
    };
  }
  
  /**
   * Prepare the editor for usage.
   * 
   * This method is synchronized because of the dump utility method. The editor 
   * might get prepared on different threads simultaneously.
   * 
   * @param context Context information. 
   * @param forEditing <code>true</code> if we prepare for editing. <code>false</code>
   * if we prepare for renderering a value.
   */
  private void prepare(AuthorInplaceContext context, boolean forEditing) {
    if (textField == null) {
      if (forEditing) {
        textField = new CCTextField();
      } else {
        // If we are just rendering a value we can just use a more lightweight text field.
        textField = new UndoableTextField();
      }
      
      defaultFont = textField.getFont();
      defaultForeground = textField.getForeground();
      Insets originalMargins = textField.getMargin();
      imposedInsets = null;
      
      if(originalMargins != null) {
        imposedInsets = (Insets) originalMargins.clone();
      } else {
    	//EXM-27442 - Maybe Nimbus LF, set some margins.
        imposedInsets = new Insets(1, 1, 1, 1);
      }
      
      if (PlatformDetector.isWin32()) {
        imposedInsets.top -= 2;
        imposedInsets.bottom += 3;
      }
    }
    
    if (forEditing) {
      // Make sure they are not forgotten,
      textField.removeKeyListener(keyListener);
      textField.removeFocusListener(focusListener);
    }
    
    Font font = (Font) context.getArguments().get(InplaceEditorArgumentKeys.FONT);
    if (font != null) {
      textField.setFont(new java.awt.Font(font.getName(), font.getStyle(), font.getSize()));
    } else {
      textField.setFont(defaultFont);
    }
    
    ro.sync.exml.view.graphics.Color color =
        (ro.sync.exml.view.graphics.Color) context.getArguments().get(InplaceEditorArgumentKeys.PROPERTY_COLOR);
    if (color != null) {
      textField.setForeground(new Color(color.getRed(), color.getGreen(), color.getBlue(), color.getAlpha()));
    } else {
      textField.setForeground(defaultForeground);
    }
    // EXM-25550 No point in initializing the CC support when we are just rendering 
    // a value.
    if (forEditing) {
      // No point in creating so many managers.
      ccManager.setContext(context);

      // Re-initialize.
      ((CCTextField) textField).setCCManager(ccManager);
    }
    
    String text = (String) context.getArguments().get(InplaceEditorArgumentKeys.INITIAL_VALUE);
    if (!forEditing && text == null) {
      text = (String) context.getArguments().get(InplaceEditorArgumentKeys.DEFAULT_VALUE);
      textField.setForeground(Color.GRAY);
    }
    
    if (text == null) {
      text = "";
    }
    
    textField.setText(text);
    
    if (forEditing) {
      // Make sure they are not forgotten,
      textField.addKeyListener(keyListener);
      textField.addFocusListener(focusListener);
      textField.getDocument().addDocumentListener(new DocumentListener() {
        @Override
        public void removeUpdate(DocumentEvent e) {
          fireEditingOccured();
        }
        @Override
        public void insertUpdate(DocumentEvent e) {
          fireEditingOccured();
        }
        @Override
        public void changedUpdate(DocumentEvent e) {
          fireEditingOccured();
        }
      });
    }
    
    if (!forEditing
        && context.getAuthorAccess() != null
        && context.getAuthorAccess().getWorkspaceAccess() != null
        && !context.getAuthorAccess().getWorkspaceAccess().isStandalone()) {
      textField.setMargin(imposedInsets);
    }
    
    String tooltip = (String) context.getArguments().get(InplaceEditorArgumentKeys.PROPERTY_TOOLTIP);
    textField.setToolTipText(tooltip);
    
    InplaceEditorUtil.relayout(textField, context);
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceRenderer#getRendererComponent(ro.sync.ecss.extensions.api.editor.AuthorInplaceContext)
   */
  @Override
  public Object getRendererComponent(AuthorInplaceContext context) {
    prepare(context, false);
    
    return textField;
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceRenderer#getRenderingInfo(ro.sync.ecss.extensions.api.editor.AuthorInplaceContext)
   */
  @Override
  public RendererLayoutInfo getRenderingInfo(AuthorInplaceContext context) {
    prepare(context, false);
    
    final Dimension preferredSize = textField.getPreferredSize();

    return new RendererLayoutInfo(
        textField.getBaseline(preferredSize.width, preferredSize.height),
        InplaceEditorUtil.getPreferredSize(textField, context));
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#getEditorComponent(ro.sync.ecss.extensions.api.editor.AuthorInplaceContext, ro.sync.exml.view.graphics.Rectangle, ro.sync.exml.view.graphics.Point)
   */
  @Override
  public Object getEditorComponent(AuthorInplaceContext context, Rectangle allocation, Point mouseLocation) {
    prepare(context, true);
    
    return textField;
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#requestFocus()
   */
  @Override
  public void requestFocus() {
    textField.requestFocus();
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#getValue()
   */
  @Override
  public Object getValue() {
    return textField.getText();
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#stopEditing()
   */
  @Override
  public void stopEditing() {
    stopEditing(true, false);
  }
  
  /**
   * Stops editing.
   * 
   * @param requestFocus <code>true</code> to request focus inside the author component.
   */
  private void stopEditing(boolean requestFocus, boolean onEnter) {
    // Avoid any other events being processed.
    textField.removeKeyListener(keyListener);
    textField.removeFocusListener(focusListener);
    
    // We don't commit anything from the popUp window.
    ((CCTextField) textField).cancelCC();
    
    if (onEnter) {
      fireNextEditLocationRequested();
    } else {
      fireEditingStopped(new EditingEvent((String)getValue(), requestFocus));
    }
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#cancelEditing()
   */
  @Override
  public void cancelEditing() {
    // Avoid any other events being processed.
    textField.removeKeyListener(keyListener);
    textField.removeFocusListener(focusListener);
    
    ((CCTextField) textField).cancelCC();
    fireEditingCanceled();
  }

  /**
   * @see ro.sync.ecss.extensions.api.Extension#getDescription()
   */
  @Override
  public String getDescription() {
    return "A text field renderer and editor";
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#getScrollRectangle()
   */
  @Override
  public Rectangle getScrollRectangle() {
    Rectangle rectangle = null;
    try {
      java.awt.Rectangle modelToView = textField.modelToView(textField.getCaretPosition());
      rectangle = new Rectangle(modelToView.x, modelToView.y, modelToView.width, modelToView.height);
    } catch (BadLocationException e) {
      // Shouldn't happen.
      logger.error(e, e);
    }
    
    return rectangle;
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceRenderer#getTooltipText(ro.sync.ecss.extensions.api.editor.AuthorInplaceContext, int, int)
   */
  @Override
  public String getTooltipText(AuthorInplaceContext context, int x, int y) {
    prepare(context, false);
    String toolTipText = textField.getToolTipText();
    if (toolTipText == null) {
      toolTipText = textField.getText();
    }
    return toolTipText;
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceRenderer#getCursorType(ro.sync.ecss.extensions.api.editor.AuthorInplaceContext, int, int)
   */
  @Override
  public CursorType getCursorType(AuthorInplaceContext context, int x, int y) {
    return CursorType.CURSOR_NORMAL;
  }

  /**
   * @see ro.sync.ecss.component.editor.LoggableInplaceRenderer#dump(ro.sync.ecss.extensions.api.editor.AuthorInplaceContext, java.lang.StringBuilder)
   */
  @Override
  public void dump(AuthorInplaceContext inplaceContext, StringBuilder builder) {
    prepare(inplaceContext, false);
    builder.append("value:'").append(textField.getText()).append("'");
    if (textField.getToolTipText() != null) {
      builder.append(", tooltip:'").append(textField.getToolTipText()).append("'");
    }
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceRenderer#getCursorType(int, int)
   */
  @Override
  public CursorType getCursorType(int x, int y) {
    return null;
  }
}
