/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.component.validation;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.annotations.obfuscate.SkipLevel;
import ro.sync.annotations.obfuscate.SkipObfuscate;
import ro.sync.document.DPIData;
import ro.sync.document.DocumentPositionedInfo;
import ro.sync.ecss.extensions.api.node.AuthorNode;
import ro.sync.exml.editor.EditorPageConstants;

/**
 * A document position info usually needs a line and column for the error.
 * This extension allows you to specify either the problem AuthorNode or an offset/length interval in the Author content.
 */
@SkipObfuscate(classes = SkipLevel.NOT_SPECIFIED, fields = SkipLevel.PUBLIC, methods = SkipLevel.PUBLIC)
@API(type=APIType.EXTENDABLE, src=SourceType.PUBLIC)
public class AuthorDocumentPositionedInfo extends DocumentPositionedInfo {
  
  /**
   * Marks in the DPI that the offset is from the Author content
   */
  public static DPIData CONTENT_DATA = new DPIData() {};

  /**
   * The node that generated the dpi. Can be null
   */
  private AuthorNode node;
  /**
   * If <code>true</code> the entire node will be selected.
   */
  private boolean selectEntireNode = true;

  /**
   * Constructor.
   * 
   * @param severity Severity. One of the severity constants from class DocumentPositionedInfo: 
   * SEVERITY_ERROR, SEVERITY_FATAL, SEVERITY_INFO , SEVERITY_WARN. 
   * @param message Error message.
   * @param systemID System ID
   * @param node  The author node.
   */
  public AuthorDocumentPositionedInfo(
      int severity, 
      String message, 
      String systemID,
      AuthorNode node) {
    super(severity, message, systemID);
    this.node = node;
    setData(CONTENT_DATA);
    setImposedInitialPage(EditorPageConstants.PAGE_AUTHOR);
  }
  
  /**
   * Constructor.
   * 
   * @param severity Severity. One of the severity constants from class DocumentPositionedInfo: 
   * SEVERITY_ERROR, SEVERITY_FATAL, SEVERITY_INFO , SEVERITY_WARN. 
   * @param message Error message.
   * @param systemID System ID
   * @param startOffset The start offset of the problem, mapped in the Author content. 
   * @param length The length of the problem, mapped in the Author content. 
   */
  public AuthorDocumentPositionedInfo(
      int severity, 
      String message, 
      String systemID,
      int startOffset,
      int length) {
    super(severity, message, systemID);
    setOffset(startOffset);
    setLength(length);
    setData(CONTENT_DATA);
    setImposedInitialPage(EditorPageConstants.PAGE_AUTHOR);
  }

  /**
   * @return The node to locate the message.
   */
  public AuthorNode getNode() {
    return node;
  }
  
  /**
   * Sets if the entire node should be selected or not.
   *
   * @param selectEntireNode <code>true</code> if the entire node should be selected.
   */
  public void setSelectEntireNode(boolean selectEntireNode) {
    this.selectEntireNode = selectEntireNode;
  }
  
  /**
   * Checks if the entire node should be selected.
   *
   * @return <code>true</code> if the entire node should be selected.
   */
  public boolean isSelectEntireNode() {
    return selectEntireNode;
  }
}