/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2009 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.extensions.api.node.AuthorElement;

/**
 * This is an interface for classes which are responsible for providing information 
 * about the cell separators: "rowsep" and "colsep".
 * 
 * It should be implemented when the author extension being developed offers
 * support for editing data in tabular form.
 * @see "http://www.docbook.org/tdg5/en/html/cals.table.html"
 * @see "http://docs.oasis-open.org/dita/v1.0/langspec/table.html"
 * @see "https://www.oasis-open.org/specs/tm9901.html"
 */
@API(type=APIType.EXTENDABLE, src=SourceType.PUBLIC)
public interface AuthorTableCellSepProvider extends Extension {
  /**
   * Checks if a separator should be placed at the cell right.  
   * Note that if the cell is the last from its row, the separator is not painted even if this
   * method returns <code>true</code>.
   * 
   * @param cellElement The node that represents a table cell in CSS.
   * @param columnIndex The index of the column, used to identify the colspec associated 
   * to the cell. The colspec can give information about the colsep. 1 based.
   * @return <code>true</code> if a separator should be placed at its right, <code>false</code> otherwise.
   */
  boolean getColSep(AuthorElement cellElement, int columnIndex);
  
  /**
   * Checks if a separator should be placed at the cell bottom.  
   * Note that if the cell is on the last row, the separator is not painted 
   * even if this method returns <code>true</code>.
   * 
   * @param cellElement The node that represents a table cell in CSS.
   * @param columnIndex The index of the column, used to identify the rowspec associated 
   * to the cell. The rowspec can give information about the colsep. 1 based.
   * @return <code>true</code> if a separator should be placed at its right, <code>false</code> otherwise.
   */
  boolean getRowSep(AuthorElement cellElement, int columnIndex);
  
  /**
   * This method is called when starting to compute the layout for a table. Its intended
   * to extract information from the element representing the table only once, not on every 
   * getColSep() or getRowSep() call. Example: for a CALS table we identify and cache
   * the <code>colsep</code> and <code>rowsep</code> elements from that table.
   * 
   * A new instance of the table cell span provider is used for every table in a document so 
   * cached data cannot be used between different tables..
   * 
   * @param tableElement The {@link AuthorElement} representing a table (it has the CSS display property 
   * set on 'table').
   */
  void init(AuthorElement tableElement);  
}