/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api.callouts;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.extensions.api.AuthorReviewController;
import ro.sync.ecss.extensions.api.highlights.AuthorPersistentHighlight;
import ro.sync.exml.view.graphics.Rectangle;

/**
 * The callouts are representations of Track Changes insert and delete highlights, 
 * review comment highlights and custom review highlights in the Author mode on a side bar.
 * <br/>
 * This controller can be used to decide what types of callouts must be presented in 
 * Author mode. It must be provided through {@link AuthorReviewController#getAuthorCalloutsController()}
 * method.
 * <br/>
 * <br/>
 * To render a custom highlight as a callout in Author mode, a callout information 
 * provider must be set from 
 * {@link AuthorCalloutsController#setCalloutsRenderingInformationProvider(CalloutsRenderingInformationProvider)} method.
 * <br/>
 * <br/>
 * By default, the callouts visibility in Author mode is controlled from Oxygen
 * Preferences but it can be changed by using the methods in this class.
 * The Preferences will not be affected by the API. 
 * 
 * @since 14
 */
@API(type=APIType.NOT_EXTENDABLE, src=SourceType.PUBLIC)
public interface AuthorCalloutsController {
  
  /**
   * Check if the callouts corresponding to review comments and Change Tracking
   * deletions and insertions with comments are visible in Author mode. 
   * <br/>
   * By default, the comments callouts visibility in Author mode is controlled from Oxygen
   * Preferences but it can be changed by using the {@link #setShowCommentsCallouts(Boolean)} 
   * method. 
   * <br/>
   * Note that when there are no review callouts, the callouts side bar is collapsed.
   * 
   * @return <code>true</code> if the callouts with comments are visible.
   */
  public boolean isShowingCommentsCallouts();
  
  /**
   * The Track Changes insert and delete markers, the review comment markers and 
   * the custom review markers can be presented in Author mode as callouts.
   * <br/>
   * This method can be used to override the default option from Oxygen Preferences 
   * that controls if the callouts corresponding to review comments and Change Tracking
   * deletions and insertions with comments are displayed in Author mode.
   * <br/>
   * Note that when there are no review callouts, the callouts side bar is collapsed.
   * 
   * @param showCommentsCallouts If <code>true</code>, the review callouts with comments
   * are displayed in Author mode. The callouts with comments are hidden when the provided 
   * value is <code>false</code>.
   * <br/> 
   * When the value is set to <code>null</code>, the option from Oxygen Preferences
   * is taken into consideration.
   */
  public void setShowCommentsCallouts(Boolean showCommentsCallouts);
  
  /**
   * Check if the callouts corresponding to Change Tracking deletions are visible in Author mode. 
   * <br/>
   * By default, the Change Tracking deletions callouts visibility in Author mode is controlled from Oxygen
   * Preferences but it can be changed by using the {@link #setShowDeletionsCallouts(Boolean)} 
   * method. 
   * <br/>
   * Note that when there are no review callouts, the callouts side bar is collapsed.
   * 
   * @return <code>true</code> if the Track Changes deletions callouts are 
   * displayed in Author mode. 
   */
  public boolean isShowingDeletionsCallouts();
  
  /**
   * The Track Changes insert and delete markers, the review comment markers and 
   * the custom review markers can be presented in Author mode as callouts.
   * <br/>
   * This method can be used to override the default option from Oxygen Preferences 
   * that controls if the callouts corresponding to Change Tracking
   * deletions are displayed in Author mode.
   * <br/>
   * Note that when there are no review callouts, the callouts side bar is collapsed.
   * 
   * @param showDeletionsCallouts If <code>true</code>, the Track Changes deletions callouts
   * are displayed in Author mode. The deletions callouts are hidden when the provided value is <code>false</code>.
   * <br/> 
   * When the value is set to <code>null</code>, the option from Oxygen Preferences
   * is taken into consideration.
   */
  public void setShowDeletionsCallouts(Boolean showDeletionsCallouts);
  
  /**
   * Check if the callouts corresponding to Change Tracking insertions are visible in Author mode. 
   * <br/>
   * By default, the Change Tracking insertions callouts visibility in Author mode is controlled from Oxygen
   * Preferences but it can be changed by using the {@link #setShowInsertionsCallouts(Boolean)} 
   * method. 
   * <br/>
   * Note that when there are no review callouts, the callouts side bar is collapsed.
   * 
   * @return <code>true</code> if the Track Changes insertions callouts are visible.
   */
  public boolean isShowingInsertionsCallouts();
  
  /**
   * The Track Changes insert and delete markers, the review comment markers and 
   * the custom review markers can be presented in Author mode as callouts.
   * <br/>
   * This method can be used to override the default option from Oxygen Preferences 
   * that controls if the callouts corresponding to Change Tracking
   * insertions are displayed in Author mode.
   * <br/>
   * Note that when there are no review callouts, the callouts side bar is collapsed.
   * 
   * @param showInsertionsCallouts If <code>true</code>, the Track Changes insertions callouts
   * are displayed in Author mode. The insertions callouts are hidden when the provided value is <code>false</code>.
   * <br/> 
   * When the value is set to <code>null</code>, the option from Oxygen Preferences
   * is taken into consideration.
   */
  public void setShowInsertionsCallouts(Boolean showInsertionsCallouts);
  
  /**
   * Set the provider for data that will be rendered as a callout, in Author mode, 
   * for a specific highlight. 
   * <br/>
   * The callouts are representations of Track Changes insert and delete highlights, 
   * review comment highlights and custom review highlights in Author mode.
   * <br/>
   * By default, the callouts visibility in Author mode is controlled from Oxygen
   * Preferences but it can be changed by using the {@link AuthorCalloutsController} methods.
   * 
   * @param provider The highlights callout rendering information provider.
   * @throws IllegalArgumentException Thrown when a property name is not a valid XML attribute name. 
   * 
   * @since 14
   */
  public void setCalloutsRenderingInformationProvider(CalloutsRenderingInformationProvider provider);
  
  /**
   * Retrieves the bounds of the callout box associated with an Author persistent highlight. 
   * 
   * @param persistentHighlight The Author persistent highlight.
   * 
   * @return The bounds of the callout box associated with the given Author persistent highlight,
   *         relative to the editor area's origins, or <code>null</code> if there is no corresponding
   *         callout box.
   * 
   * @since 14.2
   */
  public Rectangle getCalloutRectangle(AuthorPersistentHighlight persistentHighlight);
}
