/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api.editor;

import java.awt.event.FocusListener;
import java.awt.event.KeyListener;

import javax.swing.event.DocumentListener;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.extensions.api.Extension;
import ro.sync.ecss.extensions.commons.editor.SimpleURLChooserEditor;
import ro.sync.ecss.extensions.commons.editor.URLChooserEditorSWT;
import ro.sync.exml.view.graphics.Point;
import ro.sync.exml.view.graphics.Rectangle;

/**
 * An author in-place editor. Subclasses contain implementation based on real
 * SWT or Swing components like combo boxes, text fields, check boxes, buttons etc.
 * 
 * <p>An editor is associated with an oxy_editor CSS function and is invoked on user interaction.
 * It only handles edit requests while the {@link InplaceRenderer} will handle the 
 * painting of the value to edit.</p>
 * <p>An editor implementation is used through the oxy_editor function like this:
 * <pre>
 * code {
 *   content: 
 *     "Attr1: " oxy_editor(type, combo, edit, '@attr1')
 *     "Attr2: " oxy_editor(type, text, edit, '@attr2')
 *  }
 * </pre>
 * In the previous example a predefined <b>combo</b> editor is requested for editing the value 
 * of attribute <b>attr1</b> and a predefined <b>text</b> editor for editing attribute <b>attr2</b>. 
 * 
 * For a custom implementation properties {@link InplaceEditorCSSConstants#PROPERTY_SWING_EDITOR_CLASS_NAME}
 * and {@link InplaceEditorCSSConstants#PROPERTY_SWT_EDITOR_CLASS_NAME} must be used:
 * <pre>
 * myElement {
 *   content: oxy_editor(
 *       rendererClassName, "com.custom.editors.CustomRenderer",
 *       swingEditorClassName, "com.custom.editors.SwingCustomEditor",
 *       swtEditorClassName, "com.custom.editors.SwtCustomEditor",
 *       edit, "@my_attr"
 *       customProperty1, "customValue1",
 *       customProperty2, "customValue2"
 *   )
 * }
 * </pre>
 * In the previous example, the editor classes com.custom.editors.SwingCustomEditor and 
 * com.custom.editors.SwtCustomEditor must be added in the Classpath of the 
 * document type associated with the edited document.
 * </p>
 * 
 * <p>It is recommended to extend the adapter class {@link InplaceEditorRendererAdapter} or
 * {@link InplaceEditorAdapter} in order to be protected from future API additions.</p>
 * 
 * <p>A SWT implementation should also implement org.eclipse.jface.text.ITextOperationTarget
 * so that it will be delegated with events of UNDO, REDO, CUT, PASTE, SELECT ALL.</p>
 * 
 * @see SimpleURLChooserEditor
 * @see URLChooserEditorSWT
 * 
 * @since 14.1
 */
@API(type=APIType.EXTENDABLE, src=SourceType.PUBLIC)
public interface InplaceEditor extends Extension {
  /**
   * Prepare and return the editor component.
   * 
   * @param context The context where the editor will be used.
   * @param allocation The bounds where the editor will be shown. This is normally
   * the bounds of the box in which the value being edited is rendered. If the 
   * editor requires to be presented in different bounds it should alter this parameter.
   * The X,Y coordinates are relative to the parent in which the editor will be added.
   * @param mouseInvocationLocation if the editor was requested using the mouse this
   * parameter represents the X,Y location where the event took place. It is relative to the parent
   * in which the editor will be added. <code>null</code> if the editor wasn't requested through 
   * mouse interaction.
   * 
   * <br/>
   * <br/>
   * <b>OBS</b>: This is the very first call received by an editor. This ensures that the 
   * editor is properly initialized for the subsequent calls (like a {@link #requestFocus()} call).
   * <br/>
   * <b>OBS</b>: An editor implementation will have to add listeners onto itself like:
   * <ul>
   *  <li>a {@link KeyListener} for handling key events like: ENTER to stop editing 
   *  (by calling {@link InplaceEditingListener#editingStopped(EditingEvent)}) 
   *  and ESCAPE to cancel it (by calling {@link InplaceEditingListener#editingCanceled()}).</li>
   *  <li>a {@link FocusListener} to stop editing when the focus is given to a component that is not 
   *  part of the editor (by calling {@link InplaceEditingListener#editingStopped(EditingEvent)}).</li>
   *  <li>a {@link DocumentListener} to fire {@link InplaceEditingListener#editingOccured()} events (If the editor has a document).</li>
   * </ul>
   * 
   * @return The component that performs the editing. For the Standalone distribution
   * this should be a java.awt.JComponent implementation. For the Eclipse plugin
   * distribution, an org.eclipse.swt.widgets.Control is expected.
   */
  Object getEditorComponent(AuthorInplaceContext context, Rectangle allocation, Point mouseInvocationLocation);
  
  /**
   * Returns a rectangle that should be made visible after the editor is shown.
   * The coordinate should be relative to the editor itself.
   * 
   * The default behavior is to make the entire editor visible but if the editor
   * is bigger than the viewport the visible part might not be the right one. For
   * example is the editor is a text field the caret might not be visible. This is
   * when this method is useful. The caret rectangle should be returned so that 
   * the part of the editor with the caret is presented.
   * 
   * @return A rectangle to be made visible or <code>null</code> to make the entire editor 
   * visible.
   */
  Rectangle getScrollRectangle();
  
  /**
   * Adds a listener to receive edit notifications:
   * - {@link InplaceEditingListener#editingCanceled()} to remove the editor and 
   *   cancel the edit operation.
   * - {@link InplaceEditingListener#editingOccured()} to signal modification in 
   *   the editor. This event marks the editor as dirty and it's value will be 
   *   committed when a {@link InplaceEditingListener#editingStopped(EditingEvent)}
   *   is received.
   * - {@link InplaceEditingListener#editingStopped(EditingEvent)} to end editing
   *   and commit it's value if needed. The value is usually committed ONLY if 
   *   a {@link InplaceEditingListener#editingOccured()} was fired. See
   *   {@link InplaceEditingListener#editingStopped(EditingEvent)} for more information.
   * @param editingListener Editing listener. 
   */
  void addEditingListener(InplaceEditingListener editingListener);
  
  /**
   * Requests focus inside the editing component.
   */
  void requestFocus();
  /**
   * Gets the value that the user entered.
   * 
   * @return The value that the user entered.
   */
  Object getValue();
  
  /**
   * Stops the editing and commits the current value. The editor should release
   * any held resources and notify {@link InplaceEditingListener#editingStopped(EditingEvent)}.
   * 
   * OBS: The current value will be committed only if at least one 
   * {@link InplaceEditingListener#editingOccured()} event was issued before this moment. 
   */
  void stopEditing();
  
  /**
   * Cancels the editing process. The editor should release
   * any held resources and notify {@link InplaceEditingListener#editingCanceled()}.
   */
  void cancelEditing();

  /**
   * Removes a listener that receives editing events.
   * 
   * @param editingListener Editing listener. 
   */
  void removeEditingListener(InplaceEditingListener editingListener);
}