package ro.sync.ecss.extensions.api.webapp;

import java.io.Reader;
import java.io.Serializable;
import java.io.Writer;
import java.util.List;
import java.util.concurrent.Callable;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.document.DocumentPositionedInfo;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorDocumentController;
import ro.sync.ecss.extensions.api.AuthorExtensionActionProvider;
import ro.sync.ecss.extensions.api.AuthorSelectionAndCaretModel;
import ro.sync.ecss.extensions.api.highlights.AuthorPersistentHighlight;
import ro.sync.ecss.extensions.api.node.AuthorNode;
import ro.sync.ecss.extensions.api.webapp.cc.ContentCompletionManager;
import ro.sync.ecss.extensions.api.webapp.formcontrols.FormControlEditingHelper;
import ro.sync.ecss.extensions.api.webapp.review.ReviewController;
import ro.sync.ecss.webapp.DPILocation;
import ro.sync.exml.editor.scenario.BaseScenario;
import ro.sync.exml.editor.xmleditor.DocumentTypeProvider;

/**
 * The model of an XML document to be edited.
 * 
 * It has the following components:
 * <ol>
 *  <li>some associated CSS files</li>
 *  <li>a schema used to validate the document and to propose elements to be 
 *  inserted in a specific context.</li>
 *  <li>a set of markers added by reviewers</li>
 *  <li>a selection model</li>
 *  <li>an undo manager that supports undoable edits</li>
 * </ol>
 * 
 * This class is a facade over all these components.
 * 
 * @since 15.1
 */
@API(type=APIType.NOT_EXTENDABLE, src=SourceType.PUBLIC)
public interface AuthorDocumentModel extends Serializable {

	/**
	 * Getter for the document controller.
	 * 
	 * The controller is used to perform undoable edits on the document content.
	 * 
	 * @return the document controller.
	 */
	public abstract AuthorDocumentController getAuthorDocumentController();

	/**
	 * Returns a renderer of the document to the specified writer.
	 * 
	 * @param out The writer.
	 * @return The HTML renderer.
	 */
	public abstract AuthorHTMLRenderer createHTMLRenderer(Writer out);

	/**
	 * Returns a reader over the document.
	 * 
	 * The reader serializes the current content of the document trying to match
	 * the indentation and line-width that were used in the document before the
	 * editing started.
	 * 
	 * @return a reader over the document.
	 */
	public abstract Reader createReader();

	/**
	 * Returns an indexer that assigned IDs to all nodes in the document.
	 * 
	 * @return The node indexer. 
	 */
	public abstract AuthorIdIndex<AuthorNode> getNodeIndexer();

	
	/**
	 * Return an indexer that assigned IDs to all markers in the document.
	 * 
	 * @return The marker indexer. 
	 */
	public abstract AuthorIdIndex<AuthorPersistentHighlight> getMarkersIndexer();

	/**
	 * TODO (EXM-27739): split it into document-specific and doctype specific CSS in order 
	 * to allow separate caching.
	 * 
	 * Returns the CSS to be applied to the HTML document.
	 * 
	 * @return The CSS to be applied to the HTML document.
	 */
	public abstract String getCssContent();

	/**
	 * Returns a review controller that can be used to perform actions on the 
	 * markers present in the document.
	 * 
	 * @return The review controller.
	 */
	public abstract ReviewController getReviewController();

	/**
	 * Returns the content completion manager that can be used to insert XML tags
	 * in the document such that the document remains valid according to its 
	 * schema.
	 * 
	 * @return The content completion manager.
	 */
	public abstract ContentCompletionManager getContentCompletionManager();

	/**
	 * Returns the selection model of the document.
	 * 
	 * @return The selection model.
	 */
	public abstract AuthorSelectionAndCaretModel getSelectionModel();
	
	
	/**
	 * A task that tries to validate the document according to its schema and 
	 * returns the list of found errors.
	 * 
	 * @return The validation task for the current document.
	 */
	public abstract Callable<List<DocumentPositionedInfo>> getValidationTask();

	
  /**
   * @return The encoding of the document in Java format.
   */
	public abstract String getEncoding();
	
	/**
	 * @return Actions support.
	 */
	public abstract AuthorExtensionActionProvider getActionsManager();
	
  /**
   * Returns the form control editing helper.
   * 
   * @return the form control editing helper.
   */
  public abstract FormControlEditingHelper getEditingHelper();
  
  /**
   * Returns the schema aware actions handler.
   * 
   * @return The schema aware actions handler.
   */
  public abstract WebappAuthorSchemaAwareActionsHandler getActionsSupport();
  
  /**
   * Returns the Autor access object.
   * 
   * @return Returns the author access object.
   */
  public abstract AuthorAccess getAuthorAccess();

  /**
   * Returns the id of the document type of the document.
   * 
   * @return The id of the document type of the document.
   */
  public abstract String getDocumentTypeId();
  
  /**
   * Get validation scenarios associated with the document.
   * 
   * @return Validation scenarios associated with the document.
   */
  public abstract List<BaseScenario> getValidationScenarios();
  
  /**
   * Get document type provider.
   * 
   * @return The document type provider.
   */
  public abstract DocumentTypeProvider getDocTypeProvider();
  
  /**
   * Compute for the given document position info the content offsets. 
   * 
   * @param dpInfo The document position info.
   *
   * @return The DPI location info.
   */
  public abstract DPILocation getDPILocation(DocumentPositionedInfo dpInfo);
}