/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.commons.table.support;

import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.extensions.api.node.AttrValue;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;

/**
 * DITA CALS table cell info provider, should work with specializations.
 */
@API(type=APIType.INTERNAL, src=SourceType.PUBLIC)
public class DITACALSTableCellInfoProvider extends CALSTableCellInfoProvider {
  
  /**
   * The thead of the table class attribute value.
   */
  private static final String THEAD_CLASS_VALUE = " topic/thead ";

  /**
   * The tgroup of the table class attribute value.
   */
  private static final String TGROUP_CLASS_VALUE = " topic/tgroup ";

  /**
   * The row of the table class attribute value.
   */
  private static final String ROW_CLASS_VALUE = " topic/row ";
  
  /**
   * The cell of the table class attribute value.
   */
  private static final String TABLE_CELL_CLASS_VALUE = " topic/entry ";
  
  /**
   * The attribute name class.
   * The value is <code>class</code>
   */
  private static final String ATTRIBUTE_NAME_CLASS = "class";
  /**
   * Gathered names of cells from the table.
   */
  private Set<String> cellTagNames = new HashSet<String>();
  /**
   * @see ro.sync.ecss.extensions.commons.table.support.CALSTableCellInfoProvider#init(ro.sync.ecss.extensions.api.node.AuthorElement)
   */
  @Override
  public void init(AuthorElement tableElement) {
    super.init(tableElement);
    cellTagNames = detectCALSCellTagNames(tableElement);
  }

  /**
   * @see ro.sync.ecss.extensions.commons.table.support.CALSTableCellInfoProvider#isTableCell(java.lang.String)
   */
  @Override
  protected boolean isTableCell(String tableCellsTagName) {
    return super.isTableCell(tableCellsTagName) || cellTagNames.contains(tableCellsTagName);
  }

  
  /**
   * Determine the list of cell tag names.
   * 
   * @param tableElement The table element.
   */
  private static Set<String> detectCALSCellTagNames(AuthorElement tableElement) {
    Set<String> cellTagNames = new  HashSet<String>();
    // Determine the table cell tag name.
    List<AuthorNode> tableChildren = tableElement.getContentNodes();
    for (Iterator<AuthorNode> iterator = tableChildren.iterator(); iterator.hasNext();) {
      //For every child of the table element.
      AuthorNode tableChild = iterator.next();
      if (tableChild.getType() == AuthorNode.NODE_TYPE_ELEMENT) {
        AttrValue classAttrVal = ((AuthorElement) tableChild).getAttribute(ATTRIBUTE_NAME_CLASS);
        //topic/thead
        //topic/tbody
        if (classAttrVal != null 
            && classAttrVal.getRawValue() != null
            //Thead or Tgroup
            && (classAttrVal.getRawValue().contains(THEAD_CLASS_VALUE)
                || classAttrVal.getRawValue().contains(TGROUP_CLASS_VALUE))) {
          List<AuthorNode> tgroupChildren = ((AuthorElement) tableChild).getContentNodes();
          //For every row node.
          for (Iterator<AuthorNode> iterator2 = tgroupChildren.iterator(); iterator2.hasNext();) {
            AuthorNode groupChild = iterator2.next();
            if (groupChild.getType() == AuthorNode.NODE_TYPE_ELEMENT) {
              classAttrVal = ((AuthorElement) groupChild).getAttribute(ATTRIBUTE_NAME_CLASS);
              if (classAttrVal != null 
                  && classAttrVal.getRawValue() != null
                  //It should be a row
                  && classAttrVal.getRawValue().contains(ROW_CLASS_VALUE)) {
                //Get the cells.
                List<AuthorNode> rowChildren = ((AuthorElement) groupChild).getContentNodes();
                for (Iterator<AuthorNode> iterator3 = rowChildren.iterator(); iterator3.hasNext();) {
                  AuthorNode rowChild = iterator3.next();
                  if (rowChild.getType() == AuthorNode.NODE_TYPE_ELEMENT) {
                    classAttrVal = ((AuthorElement) rowChild).getAttribute(ATTRIBUTE_NAME_CLASS);
                    if (classAttrVal != null 
                        && classAttrVal.getRawValue() != null 
                        && classAttrVal.getRawValue().contains(TABLE_CELL_CLASS_VALUE)) {
                      //Found a cell, learn its name.
                      cellTagNames.add(((AuthorElement) rowChild).getLocalName());
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
    return cellTagNames;
  }
}
