/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.dita;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.contentcompletion.xml.CIAttribute;
import ro.sync.contentcompletion.xml.CIElement;
import ro.sync.contentcompletion.xml.CIValue;
import ro.sync.contentcompletion.xml.Context;
import ro.sync.contentcompletion.xml.StyleGuideSchemaManagerFilterBase;
import ro.sync.contentcompletion.xml.WhatAttributesCanGoHereContext;
import ro.sync.contentcompletion.xml.WhatElementsCanGoHereContext;
import ro.sync.contentcompletion.xml.WhatPossibleValuesHasAttributeContext;
import ro.sync.ecss.dita.DITAAccess;

/**
 * Schema manager filter which provides the available keyref + condition values.
 */
@API(type=APIType.INTERNAL, src=SourceType.PUBLIC)
public class DITASchemaManagerFilter extends StyleGuideSchemaManagerFilterBase {
  
  /**
   * Description for outputclass attribute values.
   */
  private static final String OUTPUTCLASS_VALUES_DESCRIPTION =
  "Provide content highlight in the DITA OT XHTML and PDF outputs for the specific language.";

  /**
   * <code>script</code> element types. 
   */
  private static final String[] FORMAT_VALUES = new String[] {
    "dita",
    "ditamap",
    "html",
    "pdf"
  };
  
  /**
   * <code>align</code> attribute values.
   */
  private static final String[] ALIGN_VALUES = new String[] {
    "left",
    "right",
    "center",
  };
  
  /**
   * <code>chunk</code> attribute values.
   */
  private static final Map<String, String> CHUNK_VALUES = new HashMap<String, String>();

  static {
    // EXM-28405 "chunk" attribute values
    CHUNK_VALUES.put("select-topic", "The \"select-topic\" token is used to select an individual topic without any ancestors, descendents, or peers from within the same document.");
    CHUNK_VALUES.put("select-document", "The \"select-document\" token is used to select the target topic together with all ancestors, descendents, and peers within the target document.");
    CHUNK_VALUES.put("select-branch", "The \"select-branch\" token is used to select the target topic together with its descendents.");
    CHUNK_VALUES.put("by-topic", "The \"by-topic\" token establishes a policy for the current topicref (or topicref specialization) where a separate output chunk is produced for each of the selected topics.");
    CHUNK_VALUES.put("by-document", "The \"by-document\" token establishes a policy for the current topicref (or topicref specialization) where a single output chunk is produced for the referenced topic or topics.");
    CHUNK_VALUES.put("to-content", "The \"to-content\" token indicates that the selection should be rendered as a new chunk of content.\n" + 
        "\n" + 
        "    - When specified on a topicref or topicref specialization, this means that the topics selected by this topicref and its children will be rendered as a single chunk of content.\n" + 
        "    - When specified on the map element, this indicates that the contents of all topics referenced by the map are to be rendered as a single document.\n" + 
        "    - When specified on a topicref or topicref specialization that contains a title but no target, this indicates that a title-only topic must be generated in the rendered result, along with any topics referenced by child topicrefs (and topicref specializations) of this topicref. The rendition address of the generated topic is determined as defined for the copy-to attribute. If the copy-to attribute is not specified and the topicref has no id attribute, the address of the generated topic is not required to be predictable or consistent across rendition instances.\n" + 
        "\n" + 
        "For cross references to topicref elements, if the value of the chunk attribute is \"to-content\" or is unspecified, the cross reference is treated as a reference to the target topic. If the reference is to a topicref with no target, it is treated as a reference to the generated title-only topic.");
    CHUNK_VALUES.put("to-navigation", "The \"to-navigation\" token indicates that a new chunk of navigation should be used to render the current selection (such as an individual Table of Contents or related links).\n" +
        "When specified on the map element, this token indicates that the map should be presented as a single navigation chunk. If a cross reference is made to a topicref that has a title but no target, and the chunk value of that topicref is set to \"to-navigation\", the resulting cross reference is treated as a reference to the rendered navigation document (such as an entry in the table of contents).");    
  }
  
  /**
   * <code>outputclass</code> attribute values.
   */
  private static final String[] OUTPUTCLASS_VALUES = new String[] {
    "language-xml",
    "language-java",
    "language-css",
    "language-javascript",
    "language-sql",
    "language-c",
    "language-cpp",
    "language-csharp",
    "language-ini",
    "language-python",
    "language-ruby",
    "language-perl",
    "language-bourne",
    "language-php"
  };

  /**
   * The document type name
   */
  private final String documentTypeName;
  
  /**
   * Constructor 
   * 
   * @param documentTypeName The document type name
   */
  public DITASchemaManagerFilter(String documentTypeName) {
    super("http://www.oxygenxml.com/dita/styleguide/contentCompletionElementsMap.xml");
    this.documentTypeName = documentTypeName;
  }

  /**
   * Filter attribute values.
   * 
   * @see ro.sync.contentcompletion.xml.SchemaManagerFilter#filterAttributeValues(java.util.List, ro.sync.contentcompletion.xml.WhatPossibleValuesHasAttributeContext)
   */
  @Override
  public List<CIValue> filterAttributeValues(List<CIValue> attributeValues,
      WhatPossibleValuesHasAttributeContext context) {
    List<CIValue> filterAttributeValues = DITAAccess.filterAttributeValues(attributeValues, context, documentTypeName, null);
    Set<String> filteredValues = new HashSet<String>();
    
    for (int i = 0; filterAttributeValues != null && i < filterAttributeValues.size(); i++) {
      filteredValues.add(filterAttributeValues.get(i).getValue());
    }
    
    // EXM-25224 Add "dita", "ditamap", "html", "pdf" values for attribute "format"
    if ("format".equals(context.getAttributeName())) {
      if (filterAttributeValues == null) {
        filterAttributeValues = new ArrayList<CIValue>();
      }
      for (String formatValue : FORMAT_VALUES) {
        if (!filteredValues.contains(formatValue)) {
          filterAttributeValues.add(new CIValue(formatValue));
        }
      }
    } else if ("align".equals(context.getAttributeName())) {
      // EXM-26710 Add "left", "center" and "right" values for attribute "align"
      if (filterAttributeValues == null) {
        filterAttributeValues = new ArrayList<CIValue>();
      }
      for (String alignValue : ALIGN_VALUES) {
        
        if (!filteredValues.contains(alignValue)) {
          filterAttributeValues.add(new CIValue(alignValue));
        }
      }
    } else if ("chunk".equals(context.getAttributeName())) {
      // EXM-28405 Add values for "chunk" attribute
      if (filterAttributeValues == null) {
        filterAttributeValues = new ArrayList<CIValue>();
      }
      
      Set<String> keySet = CHUNK_VALUES.keySet();
      for (Iterator iterator = keySet.iterator(); iterator.hasNext();) {
        String chunkValue = (String) iterator.next();
        if (!filteredValues.contains(chunkValue)) {
        filterAttributeValues.add(new CIValue(chunkValue,
            // Add also annotation for every value
            CHUNK_VALUES.get(chunkValue)));
        }
      }
    } else if ("outputclass".equals(context.getAttributeName())) {
      String elemClazz = DITAAccess.computeElementClazz(context);
      boolean isCBOrPre = elemClazz != null && elemClazz.contains(" topic/pre ");
      if (isCBOrPre) {
        // EXM-29154 Add values for "outputclass" attribute
        if (filterAttributeValues == null) {
          filterAttributeValues = new ArrayList<CIValue>();
        }

        for (String outputclassValue : OUTPUTCLASS_VALUES) {
          if (!filteredValues.contains(outputclassValue)) {
            filterAttributeValues.add(new CIValue(outputclassValue,
                OUTPUTCLASS_VALUES_DESCRIPTION));
          }
        }
      }
    }
    
    return filterAttributeValues;
  }

  /**
   * Filter attributes.
   * 
   * @see ro.sync.contentcompletion.xml.SchemaManagerFilter#filterAttributes(java.util.List, ro.sync.contentcompletion.xml.WhatAttributesCanGoHereContext)
   */
  @Override
  public List<CIAttribute> filterAttributes(List<CIAttribute> attributes,
      WhatAttributesCanGoHereContext context) {
    List<CIAttribute> attrs = DITAAccess.annotateAttributes(attributes);
    attrs = super.filterAttributes(attrs, context);
    return attrs;
  }

  /**
   * Filter element values.
   * 
   * @see ro.sync.contentcompletion.xml.SchemaManagerFilter#filterElementValues(java.util.List, ro.sync.contentcompletion.xml.Context)
   */
  @Override
  public List<CIValue> filterElementValues(List<CIValue> elementValues, Context context) {
    return elementValues;
  }

  /**
   * Filter elements.
   * 
   * @see ro.sync.contentcompletion.xml.SchemaManagerFilter#filterElements(java.util.List, ro.sync.contentcompletion.xml.WhatElementsCanGoHereContext)
   */
  @Override
  public List<CIElement> filterElements(List<CIElement> elements,
      WhatElementsCanGoHereContext context) {
    List<CIElement> elems = DITAAccess.filterElements(elements, context);
    elems = super.filterElements(elems, context);
    return elems;
  }

  /**
   * @see ro.sync.ecss.extensions.api.Extension#getDescription()
   */
  @Override
  public String getDescription() {
    return "DITA Schema Manager Filter";
  }
}