/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2009 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.dita.topic.table;

import java.util.List;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperation;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.AuthorOperationStoppedByUserException;
import ro.sync.ecss.extensions.api.node.AttrValue;
import ro.sync.ecss.extensions.api.node.AuthorDocumentFragment;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;
import ro.sync.ecss.extensions.commons.table.operations.AuthorTableHelper;
import ro.sync.ecss.extensions.commons.table.operations.InsertTableCellsContentConstants;
import ro.sync.ecss.extensions.commons.table.operations.InsertTableOperationBase;
import ro.sync.ecss.extensions.commons.table.operations.TableCustomizerConstants;
import ro.sync.ecss.extensions.commons.table.operations.TableCustomizerConstants.ColumnWidthsType;
import ro.sync.ecss.extensions.commons.table.operations.TableInfo;
import ro.sync.ecss.extensions.commons.table.operations.TableOperationsUtil;

/**
 * Operation used to insert a DITA table.
 */
@API(type=APIType.INTERNAL, src=SourceType.PUBLIC)
public class InsertTableOperation implements AuthorOperation, InsertTableOperationBase, 
  InsertTableCellsContentConstants {
  
  /**
   * The fragment that must be introduced in the table cells
   */
  protected String cellContent = "";
  
  /**
   * @see ro.sync.ecss.extensions.api.AuthorOperation#doOperation(ro.sync.ecss.extensions.api.AuthorAccess, ro.sync.ecss.extensions.api.ArgumentsMap)
   */
  @Override
  public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
      throws IllegalArgumentException, AuthorOperationException {
    Object cellFragmentObj =  args.getArgumentValue(CELL_FRAGMENT_ARGUMENT_NAME);
    if (cellFragmentObj instanceof String) {
      cellContent = (String) cellFragmentObj;
    } 
    if (cellContent == null) {
      cellContent = "";
    }
    insertTable(null, false, authorAccess, null, null);
  }

  /**
   * Add table body.
   * 
   * @param tableXMLFragment  The string buffer where to append content.
   * @param tableInfo         The table information.
   * @param fragments         The cell fragments.
   * @param cellFragments     <code>true</code> if cells.
   * @param namespace         The table namespace.
   * @param authorAccess      The Author access.
   * @param tableHelper       The table helper.
   * 
   * @throws AuthorOperationException 
   */
  private void addTableBody(
      StringBuilder tableXMLFragment,
      TableInfo tableInfo,
      AuthorDocumentFragment[] fragments,
      boolean cellsFragments,
      String namespace,
      AuthorAccess authorAccess,
      AuthorTableHelper tableHelper) throws AuthorOperationException {

    tableXMLFragment.append("<tbody>");
    for (int i = 0; i < tableInfo.getRowsNumber(); i++) {
      tableXMLFragment.append("<row>");
      for (int j = 0; j < tableInfo.getColumnsNumber(); j++) {
        if (fragments != null) {
          String cellXMLFragment = TableOperationsUtil.createCellXMLFragment(
              authorAccess, 
              fragments, 
              cellsFragments, 
              "entry", 
              tableInfo.isGenerateHeader() ? (i +1) : i, 
              namespace, 
              tableHelper);
          tableXMLFragment.append(cellXMLFragment);
        } else {
          tableXMLFragment.append("<entry>");
          tableXMLFragment.append(cellContent);
          tableXMLFragment.append("</entry>");
        }
      }
      tableXMLFragment.append("</row>"); 
    }
    tableXMLFragment.append("</tbody>");
  }

  /**
   * Add table header.
   *
   * @throws AuthorOperationException 
   */
  private void addTableHeader(StringBuilder tableXMLFragment, TableInfo tableInfo, 
      AuthorDocumentFragment[] fragments, boolean cellsFragments, String namespace, 
      AuthorAccess authorAccess, AuthorTableHelper tableHelper) throws AuthorOperationException {
    if (tableInfo.isGenerateHeader()) {
      tableXMLFragment.append("<thead><row>");
      for (int i = 1; i <= tableInfo.getColumnsNumber(); i++) {
        if (i == 1 && fragments != null && fragments.length > 0) {
          String cellXMLFragment = TableOperationsUtil.createCellXMLFragment(
              authorAccess, 
              fragments, 
              cellsFragments, 
              "entry", 
              0, 
              namespace, 
              tableHelper);
          tableXMLFragment.append(cellXMLFragment);
        } else {
          tableXMLFragment.append("<entry>");
          tableXMLFragment.append(cellContent);
          tableXMLFragment.append("</entry>");
        }
      }
      tableXMLFragment.append("</row></thead>");
    }
  }
  
  /**
   * Add simple table body.
   * 
   * @param tableXMLFragment  The string buffer to append content.
   * @param tableInfo         The table info.
   * @param fragments         The cell fragments.
   * @param cellFragments     <code>true</code> if cells.
   * @param namespace         The table namespace.
   * @param authorAccess      The Author access.
   * @param tableHelper       The table helper.
   *
   * @throws AuthorOperationException 
   */
  private void addSimpleTableBody(
      StringBuilder tableXMLFragment,
      TableInfo tableInfo,
      AuthorDocumentFragment[] fragments,
      boolean cellsFragments,
      String namespace,
      AuthorAccess authorAccess,
      AuthorTableHelper tableHelper) throws AuthorOperationException {

    for (int i = 0; i < tableInfo.getRowsNumber(); i++) {
      tableXMLFragment.append("<strow>");
      for (int j = 0; j < tableInfo.getColumnsNumber(); j++) {
        if (fragments != null) {
          String cellXMLFragment = TableOperationsUtil.createCellXMLFragment(
              authorAccess, 
              fragments, 
              cellsFragments, 
              "stentry", 
              tableInfo.isGenerateHeader() ? (i + 1) : i, 
              namespace, 
              tableHelper);
          tableXMLFragment.append(cellXMLFragment);
        } else {
          tableXMLFragment.append("<stentry>");
          tableXMLFragment.append(cellContent);
          tableXMLFragment.append("</stentry>");
        }
      }
      tableXMLFragment.append("</strow>"); 
    }
  }
  
  /**
   * Add choice table body.
   * 
   * @param tableXMLFragment  The string buffer to append content.
   * @param tableInfo         The table info.
   * @param fragments         The cell fragments.
   * @param cellsFragments    <code>true</code> if cells.
   * @param namespace         The table namespace.
   * @param authorAccess      The Author access.
   * @param tableHelper       The table helper.
   *
   * @throws AuthorOperationException 
   */
  private void addChoiceTableBody(
      StringBuilder tableXMLFragment,
      TableInfo tableInfo,
      AuthorDocumentFragment[] fragments,
      boolean cellsFragments,
      String namespace,
      AuthorAccess authorAccess,
      AuthorTableHelper tableHelper) throws AuthorOperationException {
    boolean isChoiceOptionColumn = false;
    if (fragments != null) {
      // See what kind of column data resides in the fragment
      List<AuthorNode> contentNodes = fragments[0].getContentNodes();
      for (AuthorNode authorNode : contentNodes) {
        if (authorNode instanceof AuthorElement) {
          AuthorElement authorElement = (AuthorElement) authorNode;
          AttrValue classAttr = authorElement.getAttribute("class");
          if (classAttr != null) {
            String value = classAttr.getValue();
            if (value.contains(" task/choptionhd ")) {
              // The fragment belongs to the first column
              isChoiceOptionColumn = true;
              break;
            } 
          }
        }
      }
    }
    
    for (int i = 0; i < tableInfo.getRowsNumber(); i++) {
      tableXMLFragment.append("<chrow>");
      if (fragments != null) {
        if (isChoiceOptionColumn) {
          String cellXMLFragment = TableOperationsUtil.createCellXMLFragment(
              authorAccess, 
              fragments, 
              cellsFragments, 
              "choption", 
              tableInfo.isGenerateHeader() ? (i + 1) : i, 
              namespace, 
              tableHelper);
          tableXMLFragment.append(cellXMLFragment);
          tableXMLFragment.append("<chdesc>");
          tableXMLFragment.append(cellContent);
          tableXMLFragment.append("</chdesc>");
        } else {
          tableXMLFragment.append("<choption>");
          tableXMLFragment.append(cellContent);
          tableXMLFragment.append("</choption>");
          String cellXMLFragment = TableOperationsUtil.createCellXMLFragment(
              authorAccess, 
              fragments, 
              cellsFragments, 
              "chdesc", 
              tableInfo.isGenerateHeader() ? (i + 1) : i, 
              namespace, 
              tableHelper);
          tableXMLFragment.append(cellXMLFragment);
        }
      } else {
        tableXMLFragment.append("<choption>");
        tableXMLFragment.append(cellContent);
        tableXMLFragment.append("</choption>");
        
        tableXMLFragment.append("<chdesc>");
        tableXMLFragment.append(cellContent);
        tableXMLFragment.append("</chdesc>");
      }
      tableXMLFragment.append("</chrow>");
    }
  }
  
  /**
   * Add simple table column widths
   */
  private void addSimpleTableColWidths(StringBuilder tableXMLFragment, TableInfo tableInfo) {
    int columnsNumber = tableInfo.getColumnsNumber();
    ColumnWidthsType columnsWidthsType = tableInfo.getColumnsWidthsType();
    if (columnsNumber > 0 && columnsWidthsType != ColumnWidthsType.DYNAMIC_COL_WIDTHS) {
      tableXMLFragment.append(" relcolwidth=\"");
      for (int i = 1; i <= columnsNumber; i++) {
        tableXMLFragment.append(TableCustomizerConstants.REL_COL_WIDTH_DEFAULT_VALUE);
        if (i != columnsNumber) {
          tableXMLFragment.append(" ");
        }
      }
      tableXMLFragment.append("\"");
    }
  }
  
  /**
   * Add simple table header.
   *
   * @throws AuthorOperationException 
   */
  private void addSimpleTableHeader(StringBuilder tableXMLFragment, TableInfo tableInfo, 
      AuthorDocumentFragment[] fragments, boolean cellsFragments, String namespace, 
      AuthorAccess authorAccess, AuthorTableHelper tableHelper) throws AuthorOperationException {
    if (tableInfo.isGenerateHeader()) {
      tableXMLFragment.append("<sthead>");
      for (int i = 1; i <= tableInfo.getColumnsNumber(); i++) {
        if (i == 1 && fragments != null && fragments.length > 0) {
          String cellXMLFragment = TableOperationsUtil.createCellXMLFragment(
              authorAccess, 
              fragments, 
              cellsFragments, 
              "stentry", 
              0, 
              namespace, 
              tableHelper);
          tableXMLFragment.append(cellXMLFragment);
        } else {
          tableXMLFragment.append("<stentry>");
          tableXMLFragment.append(cellContent);
          tableXMLFragment.append("</stentry>");
        }
      }
      tableXMLFragment.append("</sthead>");
    }
  }
  
  /**
   * Add choice table header.
   * 
   * @param tableXMLFragment  The string buffer to append content.
   * @param tableInfo         The table info.
   * @param fragments         The cell fragments.
   * @param cellsFragments    <code>true</code> if cells.
   * @param namespace         The table namespace.
   * @param authorAccess      The Author access.
   * @param tableHelper       The table helper.
   *
   * @throws AuthorOperationException 
   */
  private void addChoiceTableHeader(
      StringBuilder tableXMLFragment, 
      TableInfo tableInfo, 
      AuthorDocumentFragment[] fragments, 
      boolean cellsFragments, 
      String namespace, 
      AuthorAccess authorAccess, 
      AuthorTableHelper tableHelper) throws AuthorOperationException {
    if (tableInfo.isGenerateHeader()) {
      tableXMLFragment.append("<chhead>");
      if (fragments != null) {
        // See what kind of column data resides in the fragment
        boolean isChoiceOptionHeader = false;
        List<AuthorNode> fragmentContentNodes = fragments[0].getContentNodes();
        for (AuthorNode authorNode : fragmentContentNodes) {
          if (authorNode instanceof AuthorElement) {
            AuthorElement authorElement = (AuthorElement) authorNode;
            AttrValue classAttr = authorElement.getAttribute("class");
            if (classAttr != null) {
              String value = classAttr.getValue();
              if (value.contains(" task/choptionhd ")) {
                isChoiceOptionHeader = true;
                break;
              }
            }
          }
        }
        
        String columnHeaderFragment = TableOperationsUtil.createCellXMLFragment(
            authorAccess, 
            fragments, 
            cellsFragments, 
            isChoiceOptionHeader ? "choptionhd": "chdeschd", 
            0, 
            namespace, 
            tableHelper);
        if (isChoiceOptionHeader) {
          tableXMLFragment.append(columnHeaderFragment);
          tableXMLFragment.append("<chdeschd>");
          tableXMLFragment.append(cellContent);
          tableXMLFragment.append("</chdeschd>");
        } else {
          tableXMLFragment.append("<choptionhd>");
          tableXMLFragment.append(cellContent);
          tableXMLFragment.append("</choptionhd>");
          tableXMLFragment.append(columnHeaderFragment);
        }
      } else {
        // New table with empty header.
        tableXMLFragment.append("<choptionhd>");
        tableXMLFragment.append(cellContent);
        tableXMLFragment.append("</choptionhd>");
        tableXMLFragment.append("<chdeschd>");
        tableXMLFragment.append(cellContent);
        tableXMLFragment.append("</chdeschd>");
      }
      tableXMLFragment.append("</chhead>");
    }
  }

  /**
   * Add column specification fragment.
   */
  private void addColSpecs(StringBuilder tableXMLFragment, TableInfo tableInfo) {
    ColumnWidthsType columnsWidthsType = tableInfo.getColumnsWidthsType();
    String colWidth = null;
    if (columnsWidthsType == ColumnWidthsType.PROPORTIONAL_COL_WIDTHS) {
      // Proportional widths
      colWidth = TableCustomizerConstants.REL_COL_WIDTH_DEFAULT_VALUE;
    } else if (columnsWidthsType == ColumnWidthsType.FIXED_COL_WIDTHS) {
      // Fixed widths
      colWidth = TableCustomizerConstants.FIXED_COL_WIDTH_DEFAULT_VALUE;
    }
    for (int i = 1; i <= tableInfo.getColumnsNumber(); i++) {
      tableXMLFragment.append("<colspec colname=\"c" + i + "\" colnum=\"" + i + "\"");
      if (colWidth != null) {
        tableXMLFragment.append(" colwidth=\"" + colWidth + "\"");
      }
      tableXMLFragment.append("/>");
    }
  }

  /**
   * No arguments. The operation will display a dialog for choosing the table attributes.
   * 
   * @see ro.sync.ecss.extensions.api.AuthorOperation#getArguments()
   */
  @Override
  public ArgumentDescriptor[] getArguments() {
    return CELL_FRAGMENT_ARGUMENT_IN_ARRAY;
  }

  /**
   * @see ro.sync.ecss.extensions.api.Extension#getDescription()
   */
  @Override
  public String getDescription() {
    return "Insert a DITA table";
  }

  /**
   * @see ro.sync.ecss.extensions.commons.table.operations.InsertTableOperationBase#insertTable(ro.sync.ecss.extensions.api.node.AuthorDocumentFragment[], boolean, ro.sync.ecss.extensions.api.AuthorAccess, java.lang.String, ro.sync.ecss.extensions.commons.table.operations.AuthorTableHelper)
   */
  @Override
  public void insertTable(AuthorDocumentFragment[] fragments, boolean cellsFragments,
      AuthorAccess authorAccess, String namespace, AuthorTableHelper tableHelper)
      throws AuthorOperationException {
    int rowsCount = 0;
    int columnsCount = 0;
    if (fragments != null) {
      rowsCount = fragments.length;
      columnsCount = 1;
    }
    // Show the 'Insert table' dialog
    TableInfo tableInfo = null;
    if(authorAccess.getWorkspaceAccess().isStandalone()) {
      tableInfo = SADITATableCustomizer.getInstance().customizeTable(authorAccess, rowsCount, columnsCount);
    } else {
      //Eclipse table customization
      tableInfo = ECDITATableCustomizer.getInstance().customizeTable(authorAccess, rowsCount, columnsCount);
    }

    if (tableInfo != null) {

      // Create the table XML fragment
      StringBuilder tableXMLFragment = new StringBuilder();
      int columnsNumber = tableInfo.getColumnsNumber();
      if (tableInfo.getTableModel() == TableInfo.TABLE_MODEL_DITA_CHOICE) {
        String simpleTableTag = "";
        String tableFrameAttr = tableInfo.getFrame();
        if (tableFrameAttr != null) {
          simpleTableTag = "<choicetable frame=\"" + tableFrameAttr + "\"";
        } else {
          simpleTableTag = "<choicetable ";
        }
        tableXMLFragment.append(simpleTableTag);
        // Add table header
        addSimpleTableColWidths(tableXMLFragment, tableInfo);
        tableXMLFragment.append(">");

        // Add table header
        addChoiceTableHeader(tableXMLFragment, tableInfo, fragments, cellsFragments,
            namespace, authorAccess, tableHelper);

        // Add table body
        addChoiceTableBody(tableXMLFragment, tableInfo, fragments, cellsFragments,
            namespace, authorAccess, tableHelper);

        tableXMLFragment.append("</choicetable>");
      } else if (tableInfo.getTableModel() == TableInfo.TABLE_MODEL_DITA_SIMPLE) {
        String simpleTableTag = "";
        // Value chosen by user for frame attribute;
        String tableFrameAttr = tableInfo.getFrame();
        // EXM-23110 If the user chose the "<unspecified>" value for Frame attribute, 
        // don't insert the frame attribute, otherwise insert it with the chosen value.
        if (tableFrameAttr != null) {
          simpleTableTag = "<simpletable frame=\"" + tableFrameAttr + "\"";
        } else {
          simpleTableTag = "<simpletable ";
        }
        tableXMLFragment.append(simpleTableTag);
        // Add table header
        addSimpleTableColWidths(tableXMLFragment, tableInfo);
        tableXMLFragment.append(">");

        // Add table header
        addSimpleTableHeader(
            tableXMLFragment, tableInfo, fragments, cellsFragments, namespace, authorAccess, tableHelper);

        // Add table body
        addSimpleTableBody(
            tableXMLFragment, tableInfo, fragments, cellsFragments, namespace, authorAccess, tableHelper);

        tableXMLFragment.append("</simpletable>");
      } else {
        // Table element
        String tableFrameAttr = tableInfo.getFrame();
        // EXM-23110 If the user chose the "<unspecified>" value for Frame attribute, 
        // don't insert the frame attribute, otherwise insert it with the chosen value.
        if (tableFrameAttr != null) {
          tableXMLFragment.append("<table frame=\"" + tableFrameAttr + "\">");
        } else {
        	tableXMLFragment.append("<table>");
        }
        if(tableInfo.getTitle() != null) {
          // Title was specified, insert a table with title
          tableXMLFragment.append("<title>" + tableInfo.getTitle() + "</title>");
        }

        // Add 'tgroup', A wrapper for the main content of a table
        tableXMLFragment.append("<tgroup cols=\"" + columnsNumber + "\">");

        // Add column specifications
        addColSpecs(tableXMLFragment, tableInfo);

        // Add table header
        addTableHeader(tableXMLFragment, tableInfo, fragments, cellsFragments, 
            namespace, authorAccess, tableHelper);

        // Add table body
        addTableBody(tableXMLFragment, tableInfo, fragments, cellsFragments, 
            namespace, authorAccess, tableHelper);

        tableXMLFragment.append("</tgroup>");

        tableXMLFragment.append("</table>");
      }

      // Insert the table 
      authorAccess.getDocumentController().insertXMLFragmentSchemaAware(
          tableXMLFragment.toString(), 
          authorAccess.getEditorAccess().getCaretOffset());

    } else {
      // User canceled the operation 
      throw new AuthorOperationStoppedByUserException("Cancelled by user");
    }
  }
}