/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2009 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.dita.topic.table;

import java.awt.Component;
import java.awt.Frame;
import java.util.List;

import javax.swing.text.BadLocationException;

import org.apache.log4j.Logger;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.contentcompletion.xml.CIAttribute;
import ro.sync.contentcompletion.xml.CIElement;
import ro.sync.contentcompletion.xml.WhatElementsCanGoHereContext;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorSchemaManager;
import ro.sync.ecss.extensions.api.node.AttrValue;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;
import ro.sync.ecss.extensions.commons.table.operations.SATableCustomizerDialog;
import ro.sync.ecss.extensions.commons.table.operations.TableCustomizer;
import ro.sync.ecss.extensions.commons.table.operations.TableInfo;
import ro.sync.ecss.extensions.dita.topic.InsertEquationOperation;

/**
 * Customize a DITA table.
 * It is used on standalone implementation.
 */
@API(type=APIType.INTERNAL, src=SourceType.PUBLIC)
public class SADITATableCustomizer extends TableCustomizer {
  
  /**
   * Logger for logging. 
   */
  private static final Logger logger = Logger.getLogger(InsertEquationOperation.class.getName());
  
  /**
   * The singleton instance.
   */
  private static SADITATableCustomizer instance;

  /**
   * Get the singleton instance.
   * @return The singleton instance.
   */
  public static SADITATableCustomizer getInstance(){
    if(instance == null){
      instance = new SADITATableCustomizer();
    }
    return instance;
  }
  
  /**
   *  Constructor.
   */
  private SADITATableCustomizer() {}
 
  /**
   * @see ro.sync.ecss.extensions.commons.table.operations.TableCustomizer#showCustomizeTableDialog(ro.sync.ecss.extensions.api.AuthorAccess, int, int)
   */
  @Override
  protected TableInfo showCustomizeTableDialog(AuthorAccess authorAccess, int predefinedRowsCount,
      int predefinedColumnsCount) {
    
    // Find out if a choice table can be inserted.
    boolean canInsertChoiceTable = false;

    try {
      int caretOffset = authorAccess.getEditorAccess().getCaretOffset();

      AuthorSchemaManager authorSchemaManager =
          authorAccess.getDocumentController().getAuthorSchemaManager();

      // Create context
      WhatElementsCanGoHereContext context =
          authorSchemaManager.createWhatElementsCanGoHereContext(caretOffset);

      List<CIElement> childrenElements = authorSchemaManager.whatElementsCanGoHere(context);

      if (childrenElements != null) {
        elementsIteration:
          for (int i = 0; i < childrenElements.size(); i++) {
            // Iterate through possible elements and obtain the attributes.
            CIElement currentElement = childrenElements.get(i);
            List<CIAttribute> attributes = currentElement.getAttributes();
            if (attributes != null) {
              for (int j = 0; j < attributes.size(); j++) {
                // Iterate through current element attributes.
                CIAttribute currentAttribute = attributes.get(j);
                String attrDefaultValue = currentAttribute.getDefaultValue();
                // Check if the class attribute is a choicetable.
                if ("class".equals(currentAttribute.getName())
                    && attrDefaultValue != null
                    && attrDefaultValue.contains(" task/choicetable ")) {
                  canInsertChoiceTable = true;
                  break elementsIteration;
                }
              }
            }
          }
      }

      if (!canInsertChoiceTable) {
        // Find out if we are in a <step> or <cmd> element and force a choicetable insertion after <cmd>.
        AuthorNode currentNode = authorAccess.getDocumentController().getNodeAtOffset(caretOffset);
        if (currentNode instanceof AuthorElement) {
          AuthorElement currentElement = (AuthorElement) currentNode;
          AttrValue classAttribute = currentElement.getAttribute("class");
          if (classAttribute != null && classAttribute.getValue() != null) {
            if (classAttribute.getValue().contains(" task/step ")) {
              // We are in a <step> element. Force choicetable insertion at the end of this element.
              authorAccess.getEditorAccess().setCaretPosition(currentElement.getEndOffset());
              canInsertChoiceTable = true;
            } else if (classAttribute.getValue().contains("task/cmd")) {
              // We are in a <cmd> element. Force choicetable insertion after this element.
              authorAccess.getEditorAccess().setCaretPosition(currentElement.getEndOffset() + 1);
              canInsertChoiceTable = true;
            }
          }
        }
      }
    } catch (BadLocationException e) {
      logger.warn(e, e);
    }

    SATableCustomizerDialog tableCustomizerDialog =
        new SADITATableCustomizerDialog(
            (Frame) authorAccess.getWorkspaceAccess().getParentFrame(),
            authorAccess.getAuthorResourceBundle(),
            predefinedRowsCount,
            predefinedColumnsCount,
            canInsertChoiceTable);
    tableCustomizerDialog.setLocationRelativeTo(
        (Component) authorAccess.getWorkspaceAccess().getParentFrame());
    return tableCustomizerDialog.showDialog(tableInfo);
  }
}