/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2009 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.dita.topic.table.simpletable;

import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.AuthorTableCellSpanProvider;
import ro.sync.ecss.extensions.api.node.AttrValue;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;
import ro.sync.ecss.extensions.api.table.operations.TableColumnSpecificationInformation;
import ro.sync.ecss.extensions.commons.table.operations.AuthorTableHelper;
import ro.sync.ecss.extensions.commons.table.operations.InsertColumnOperationBase;
import ro.sync.ecss.extensions.commons.table.operations.InsertTableCellsContentConstants;
import ro.sync.ecss.extensions.commons.table.operations.TableCustomizerConstants;

/**
 * Operation used to insert a DITA simple table column.
 */
@API(type=APIType.INTERNAL, src=SourceType.PUBLIC)
public class InsertColumnOperation extends InsertColumnOperationBase implements 
  SimpleTableConstants, InsertTableCellsContentConstants {
  
  /**
   * Operation arguments.
   */
  private ArgumentDescriptor[] arguments;
  
  /**
   * Constructor.
   */
  public InsertColumnOperation() {
    this(new DITASimpleTableDocumentTypeHelper());
    ArgumentDescriptor[] superArgs = super.getArguments();
    if (superArgs != null) {
      this.arguments = new ArgumentDescriptor[superArgs.length + 1];
      // Add descriptors from super
      for (int i = 0; i < superArgs.length; i++) {
        this.arguments[i] = superArgs[i];
      }
      // Add cell fragment argument
      this.arguments[superArgs.length] = CELL_FRAGMENT_ARGUMENT;
    } else {
      this.arguments = CELL_FRAGMENT_ARGUMENT_IN_ARRAY;
    }
  }
  
  /**
   * The fragment that must be introduced in the table cells
   */
  protected String cellContent;
  
  /**
   * @see ro.sync.ecss.extensions.commons.table.operations.InsertColumnOperationBase#doOperation(ro.sync.ecss.extensions.api.AuthorAccess, ro.sync.ecss.extensions.api.ArgumentsMap)
   */
  @Override
  public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
      throws IllegalArgumentException, AuthorOperationException {
    Object cellFragmentObj =  args.getArgumentValue(CELL_FRAGMENT_ARGUMENT_NAME);
    if (cellFragmentObj instanceof String) {
      cellContent = (String) cellFragmentObj;
    } 
    if ("".equals(cellContent)) {
      cellContent = null;
    }
    super.doOperation(authorAccess, args);
  }

  /**
   * Constructor.
   * 
   * @param documentTypeHelper Document type helper, has methods specific to a document type.
   */
  protected InsertColumnOperation(AuthorTableHelper documentTypeHelper) {
    super(documentTypeHelper);
  }

  /**
   * @see ro.sync.ecss.extensions.commons.table.operations.InsertColumnOperationBase#getCellElementName(ro.sync.ecss.extensions.api.node.AuthorElement, int)
   */
  @Override
  protected String getCellElementName(AuthorElement rowElement, int newColumnIndex) {
    String rowName = rowElement.getLocalName();
    if (ELEMENT_NAME_ROW_SIMPLETABLE.equals(rowName)
      // simpletable row
        || ELEMENT_NAME_HEADER_SIMPLETABLE.equals(rowName)) {
      return ELEMENT_NAME_ENTRY_SIMPLETABLE;
    } else if (ELEMENT_NAME_ROW_CHOICETABLE.equals(rowName)) {
      // chioicetable row
      return getCellName(
          rowElement, 
          new String[] {ELEMENT_NAME_CHOPTION_CHOICETABLE, ELEMENT_NAME_CHDESC_CHOICETABLE}, 
          newColumnIndex);
    } else if (ELEMENT_NAME_HEADER_CHOICETABLE.equals(rowName)) {
      // choicetable header row
      return getCellName(
          rowElement, 
          new String[] {ELEMENT_NAME_CHOPTIONHD_CHOICETABLE, ELEMENT_NAME_CHDESCHD_CHOICETABLE}, 
          newColumnIndex);
    } else if (ELEMENT_NAME_ROW_PROPERTIES.equals(rowName)) {
      // properties table row
      return getCellName(
          rowElement, 
          new String[] {ELEMENT_NAME_PROPTYPE_PROPERTIES, ELEMENT_NAME_PROPVALUE_PROPERTIES, ELEMENT_NAME_PROPDESC_PROPERTIES}, 
          newColumnIndex);
    } else if (ELEMENT_NAME_HEADER_PROPERTIES.equals(rowName)) {
      // properties table header row
      return getCellName(
          rowElement, 
          new String[] {ELEMENT_NAME_PROPTYPEHD_PROPERTIES, ELEMENT_NAME_PROPVALUEHD_PROPERTIES, ELEMENT_NAME_PROPDESCHD_PROPERTIES}, 
          newColumnIndex);
    }
    return null;
  }
  
  /**
   * Get the name of the next cell that can be inserted
   * 
   * @param rowElement      The row element to insert the cell into.
   * @param possibleValues  The possible cell names.
   * @param newColumnIndex  The index where the cell can be inserted.
   * @return The cell name.
   */
  private String getCellName(AuthorElement rowElement, String[] possibleValues, int newColumnIndex) {
    String nextCellName = null;
    int cellIdx = 0;
    List<AuthorNode> children = rowElement.getContentNodes();
    if (!children.isEmpty()) {
      for (Iterator<AuthorNode> iterator = children.iterator(); iterator.hasNext();) {
        AuthorNode child = iterator.next();
        // Analyze the cell elements of the given row to see the name of the cell 
        // that can be inserted at the provided position
        if (child.getType() == AuthorNode.NODE_TYPE_ELEMENT) {
          String cellName = ((AuthorElement) child).getLocalName();
          // Found the cell before the inserted one.  
          if (cellIdx == newColumnIndex - 1) {
            for (int i = 0; i < possibleValues.length; i++) {
              if (possibleValues[i].equals(cellName)) {
                if (i < possibleValues.length - 1) {
                  nextCellName = possibleValues[i + 1];
                  break;
                }
              }
            }
          } else if (nextCellName != null) {
            if (cellName.equals(nextCellName)) {
              nextCellName = null;
              break;
            }
          }
          cellIdx ++;
        }
      }
    } else {
      // The row has no cell.
      nextCellName = possibleValues[0];
    }
    return nextCellName;
  }
  
  /**
   * @see ro.sync.ecss.extensions.commons.table.operations.InsertColumnOperationBase#getDefaultContentForEmptyCells()
   */
  @Override
  protected String getDefaultContentForEmptyCells() {
    return cellContent;
  }

  /**
   * @see ro.sync.ecss.extensions.commons.table.operations.InsertColumnOperationBase#getArguments()
   */
  @Override
  public ArgumentDescriptor[] getArguments() {
    return arguments;
  }
  
  /**
   * @see ro.sync.ecss.extensions.commons.table.operations.InsertColumnOperationBase#updateColumnCellsSpan(ro.sync.ecss.extensions.api.AuthorAccess, ro.sync.ecss.extensions.api.AuthorTableCellSpanProvider, ro.sync.ecss.extensions.api.node.AuthorElement, int, ro.sync.ecss.extensions.api.table.operations.TableColumnSpecificationInformation, java.lang.String)
   */
  @Override
  protected void updateColumnCellsSpan(AuthorAccess authorAccess,
      AuthorTableCellSpanProvider tableSupport, AuthorElement tableElem, int newColumnIndex,
      TableColumnSpecificationInformation columnSpecification, String namespace)
      throws AuthorOperationException {
    super.updateColumnCellsSpan(authorAccess, tableSupport, tableElem, newColumnIndex,
        columnSpecification, namespace);
    if(ELEMENT_NAME_CHOICETABLE.equals(tableElem.getLocalName())) {
      //Choice tables have 2 columns inserted from the very beginning.
      return;
    }
    //EXM-25639 And also add a new span spec for it.
    AttrValue relColWidth = tableElem.getAttribute("relcolwidth");
    if(relColWidth != null) {
      //We need to update the current value to contain a "*" also for the newly added column.
      String currentValue = relColWidth.getValue();
      //Sometimes we might now be able to update the current value.
      boolean updatedValue = false;
      int valuesCounter = 0;
      StringBuilder newValue = new StringBuilder();
      StringTokenizer st = new StringTokenizer(currentValue);
      while(st.hasMoreElements()) {
        if(valuesCounter == newColumnIndex) {
          //Found the insertion place.
          updatedValue = true;
          newValue.append(TableCustomizerConstants.REL_COL_WIDTH_DEFAULT_VALUE + " ");
        }
        //Just add the existing token.
        newValue.append(st.nextToken());
        if(st.hasMoreElements()) {
          newValue.append(" ");
        }
        valuesCounter ++;
      }
      //Or maybe the last one to be inserted
      if(valuesCounter == newColumnIndex) {
        newValue.append(" " + TableCustomizerConstants.REL_COL_WIDTH_DEFAULT_VALUE);
        updatedValue = true;
      }
      
      if(updatedValue) {
        //We can set the new value
        authorAccess.getDocumentController().setAttribute(
            "relcolwidth", new AttrValue(newValue.toString()), tableElem);
      }
    }
  }
}