/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.net.protocol;

import java.io.IOException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLStreamHandler;
import java.net.URLStreamHandlerFactory;
import java.util.Iterator;
import java.util.List;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.exml.plugin.PluginDescriptor;
import ro.sync.exml.plugin.PluginDescriptor.PluginExtensionDescription;
import ro.sync.exml.plugin.PluginExtension;
import ro.sync.exml.plugin.PluginManager;
import ro.sync.exml.plugin.urlstreamhandler.URLStreamHandlerPluginExtension;

/**
 * The URLStreamHandlerFactory that handles all the protocols supported by 
 * Oxygen. 
 * 
 * It handles both builtin protocols and plugin-contributed ones.
 * 
 * @author costi
 */
@API(type=APIType.NOT_EXTENDABLE, src=SourceType.PUBLIC)
public class OxygenURLStreamHandlerFactory implements
    URLStreamHandlerFactory {

  /**
   * @see java.net.URLStreamHandlerFactory#createURLStreamHandler(java.lang.String)
   */
  @Override
  @SuppressWarnings("deprecation")
  public URLStreamHandler createURLStreamHandler(String protocol) {
    if (protocol.equals("file")) {
      URLStreamHandler handler = new ro.sync.net.protocol.file.Handler();
      //              if (logger.isDebugEnabled()) {
      //                logger.debug("Returning " + handler + " for: " + protocol);
      //              }
      return handler;
    }
    else if (protocol.equals("http")) {
      URLStreamHandler handler = new ro.sync.net.protocol.http.Handler();
      //              if (logger.isDebugEnabled()) {
      //                logger.debug("Returning " + handler + " for: " + protocol);
      //              }
      return handler;
    } 
    else if (protocol.equals("ftp")) {
      URLStreamHandler handler = new ro.sync.net.protocol.ftp.Handler();
      //             if (logger.isDebugEnabled()) {
      //               logger.debug("Returning " + handler + " for: " + protocol);
      //             }
      return handler;
    } 
    else if (protocol.equals("sftp")) {
      URLStreamHandler handler = new ro.sync.net.protocol.sftp.Handler();
      //             if (logger.isDebugEnabled()) {
      //               logger.debug("Returning " + handler + " for: " + protocol);
      //             }
      return handler;
    } else if (protocol.equals("https")) {
      URLStreamHandler handler = new ro.sync.net.protocol.https.Handler();
      //            System.out.println("Returning " + handler + " for: " + protocol);
      return handler;
    } else if(protocol.equals("svn") || protocol.equals("svn+ssh")) {
      // URL stream handler for SVN like URLs.
      URLStreamHandler handler = new URLStreamHandler() {
        @Override
        protected URLConnection openConnection(URL u) throws IOException {
          throw new IOException("Unsupported read/write operations over svn protocols");
        }
      };
      return handler;
    } else if(protocol.equals("zip")) {
      URLStreamHandler handler = new ro.sync.net.protocol.zip.Handler();
      //             if (logger.isDebugEnabled()) {
      //               logger.debug("Returning " + handler + " for: " + protocol);
      //             }
      return handler;
    } else if (protocol.equals("oxygen")) {
      // URL stream handler for native xml or relational databases. (oXygen datasources)
      URLStreamHandler handler = new ro.sync.net.protocol.nxd.Handler();
      return handler;
    } else if (protocol.equals("dctm")) {
      // URL stream handler for native xml or relational databases. (oXygen datasources)
      URLStreamHandler handler = new ro.sync.net.protocol.nxd.DCTMHandler();
      return handler;
    } else if (protocol.equals("data")) {
      URLStreamHandler handler = new ro.sync.net.protocol.data.Handler();
      return handler;
    }  else if(! PluginManager.pluginsUpdateInProgress()){
      //EXM-10691 If the plugin manager is updating the plugins
      //no need to try to instantiate it. This could lead to recursivity.
      List plugins = PluginManager.getInstance().getPlugins();
      
      for (Iterator iter = plugins.iterator(); iter.hasNext();) {
        ro.sync.exml.plugin.Plugin plugin = (ro.sync.exml.plugin.Plugin) iter.next();
        PluginDescriptor descriptor = plugin.getDescriptor();
        List<PluginExtensionDescription> extensions = descriptor.getExtensions(PluginDescriptor.URL_HANDLER);
        if(extensions.isEmpty()) {
          extensions = descriptor.getExtensions(PluginDescriptor.URL_STREAM_HANDLER);
        }
        for (int i = 0; i < extensions.size(); i++) { 
          PluginExtension extension = extensions.get(i).pluginExtension;
          if (extension instanceof URLStreamHandlerPluginExtension) {
            URLStreamHandlerPluginExtension ush = (URLStreamHandlerPluginExtension) extension;
            URLStreamHandler handler = ush.getURLStreamHandler(protocol);
            if (handler != null) {
              // The plugin knows to handle this protocol.
              return handler;                    
            }
          }
        }
      }                            
    }
    // Not known.
    return null;
  }
}